// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

// Get hardware device information.

#ifndef SERVICE_DEVICE_UTIL_H_
#define SERVICE_DEVICE_UTIL_H_

#include <QString>
#include <QStringList>

namespace netease {

// Get cpu model name, by reading /proc/cpuinfo.
QString GetCPUModelName();

// Get machine specific id.
// That id is constructed with cpu-model-name, total-memory and mac-address.
// If all of those fields are empty, returns current timestamp.
QString GetDeviceId();

// Get filesystem statistics.
// |path| is the pathname of any file/folder within the mounted filesystem.
// Returns true if system call succeeded, false otherwise.
bool GetDiskUsage(const QString& path, qint64* total_size, qint64* freespace);

// Get name of linux distribution.
QString GetDistroName();

// Returns name and version of diestribution.
QString GetDistroNameAndVersion();

// Get physical mac address.
// Check all available network devices first.
// Then get its mac address, if that address is invalid, continue.
QString GetMacAddress();

// Get physical mac address of device with name |device_name|.
QString GetMacAddress(const QString& device_name);

// Get memory size of this device, by reading /proc/mem.
QString GetMemoryTotal();

// Get network device names.
QStringList GetNetworkDeviceList();

// Returns content of /proc/version.
QString GetLinuxVersion();

// Check if current desktop environment is Ubuntu Unity or Gnome (17.10).
bool IsUbuntuUnity();

}  // namespace netease

#endif  // SERVICE_DEVICE_UTIL_H_
