// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#ifndef BASE_FILE_UTIL_H_
#define BASE_FILE_UTIL_H_

#include <QDir>
#include <QString>

namespace netease {

// Create a folder with |folder_name| in |parent_dir| directory and
// returns a QDir object refering to its absolute path.
QDir ConcateDir(const QDir& parent_dir, const QString& folder_name);

// Create parent folders if not exist.
bool CreateParentDirs(const QString& filepath);

// Returns final component of |filepath|.
QString GetFileBasename(const QString& filepath);

// Returns lower cased extension name of |filepath|, might be empty.
QString GetFileExtension(const QString& filepath);

// Get size of file. If file not found or has no access, returns 0.
qint64 GetFileSize(const QString& filepath);

// Calculate url hash and returns its hash string.
QString GetUrlHash(const QString& url);

// Get QDir which refers to cache directory of application.
// That is $HOME/.cache/netease-cloud-music.
QDir GetUserCacheDir();

// Get QDir which refers to user config folder.
// That is $HOME/.config.
QDir GetConfigDir();

// Get QDir which refers to config folder of application.
// That is $HOME/.config/netease-cloud-music.
QDir GetUserConfigDir();

// Check if file at |path| is a media file.
bool IsMediaFile(const QString& path);

// Read contents of file, returns an empty string if failed.
// DEPRECATED: call ReadTextFile() instead.
QString ReadTextFileContent(const QString& path);

// Read text file encoded in GB18030.
QString ReadGBKTextFileContent(const QString& path);

// Read contents of file, returns true if succeeded, or false otherwise.
// |content| holds the content of that file.
bool ReadTextFile(const QString& path, QString& content);

// Like used in file managers, rename |filepath| if it is already exited.
QString RenameDuplicatedFile(const QString& filepath);

// Write content to file, returns true if succeeded, or false otherwise.
bool WriteTextFile(const QString& path, const QString& content);

}  // namespace netease

#endif  // BASE_FILE_UTIL_H_
