/*
 * Copyright (C) 2017 ~ 2017 Deepin Technology Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "base/process_util.h"

#include <signal.h>
#include <sys/types.h>
#include <unistd.h>

#include <QDebug>
#include <QDir>
#include <QHash>

#include "base/file_util.h"

namespace netease {

QVector<pid_t> GetChildProcesses(pid_t parent) {
  QHash<pid_t, pid_t> s;
  const auto dirs = QDir("/proc").entryInfoList({"[0-9]*"}, QDir::Dirs);
  for (const QFileInfo& info : dirs) {
    const QString stat_file = QDir(info.filePath()).filePath("stat");
    if (!QFile::exists(stat_file)) {
      continue;
    }
    const QString content = ReadTextFileContent(stat_file);
    if (content.isEmpty()) {
      continue;
    }

    const QStringList items = content.split(' ');
    if (items.length() > 5) {
      const pid_t pid = static_cast<pid_t>(items.at(0).toLong());
      const pid_t ppid = static_cast<pid_t>(items.at(3).toLong());
      s.insert(pid, ppid);
    }
  }

  // Find processes who ppid is |parent|.
  const pid_t kInitPid = 0;
  QVector<pid_t> result;
  for (auto iter = s.constBegin(); iter != s.constEnd(); iter++) {
    pid_t ppid = iter.value();
    while (ppid != kInitPid && ppid != parent) {
      ppid = s.value(ppid);
    }
    if (ppid == parent) {
      result.append(iter.key());
    }
  }

  return result;
}

void KillChildProcesses(pid_t parent) {
  const auto children = GetChildProcesses(parent);
  for (const auto pid : children) {
    // No need to catch error.
    kill(pid, SIGTERM);
  }
}

void KillAllProcesses() {
  const pid_t pid = getpid();
  KillChildProcesses(pid);
  kill(pid, SIGTERM);
}

}  // namespace netease