// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#ifndef CHANNEL_ORCA_REPLY_H_
#define CHANNEL_ORCA_REPLY_H_

#include <QNetworkReply>
#include <QNetworkAccessManager>
class QNetworkRequest;

namespace netease {

// Handles orpheus requests.
class OrcaReply : public QNetworkReply {
  Q_OBJECT

 public:
  OrcaReply(const QNetworkRequest& request,
            const QNetworkAccessManager::Operation operation,
            QObject* parent);

  // Reimplement from QNetworkRequest.
  void abort() override;
  qint64 bytesAvailable() const override;
  void close() override;
  bool isSequential() const override;

  // Reimplement from QIODevice.
  qint64 size() const override;

  // Convert orca uri (orpheus://orpheus/xxx) to qrc uri.
  static QString OrcaToQrc(const QUrl& url);

 protected:
  // Reimplement from QIODevice.
  qint64 readData(char* data, qint64 max_size) override;

 private:
  // Handles 404, not found error.
  void handle404();

  // Handles /cache/ requests.
  void handleCache();

  // Handles /orpheus/ requests.
  void handleOrpheus();

  // Handles /skin/ requests.
  void handleSkin();

  // Handles /orpheus/storage/ requests.
  void handleStorage();

  // Send file with path |filepath| to browser.
  void sendFile(const QString& filepath);

  // To store contents of reply.
  QByteArray content_;

  // Offset of readed data.
  qint64 offset_;

 private slots:
  // Handles StorageManager::onAlbumCoverDownloaded().
  void handleAlbumCoverDownloaded(const QString& url, const QString& filepath);

  // Handles StorageManager::onAlbumCoverError().
  void handleAlbumCoverError(const QString& url, const QString& filepath);

  // Route all requests.
  void route();
};

}  // namespace netease

#endif  // CHANNEL_ORCA_REPLY_H_
