// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#ifndef DBUS_MPRIS_ADAPTER_H_
#define DBUS_MPRIS_ADAPTER_H_

#include <QObject>
#include <QUrl>

#include "third_party/qtmpris/mprisplayer.h"

namespace netease {

class MprisAdapter : public QObject {
  Q_OBJECT
 public:
  explicit MprisAdapter(QObject* parent = nullptr);
  ~MprisAdapter() override;

  static const char kServiceName[];
  static const char kServicePath[];
  static const char kRootInterface[];
  static const char kPlayerInterface[];

 private:
  void initPlayerDBus();

  QUrl album_url_;
  MprisPlayer* mpris_player_ = nullptr;

 private slots:
  void handleAlbumCoverUpdated(const QString& filepath);

  void handlePlayerReset();
  void handlePlayerMediaChanged();
  void handlePlayerStateChanged();
  void handlePositionChanged(qlonglong position);
  void handleVolumeChanged(int volume);

  void onSeekRequested(qint64 offset);
  void onOpenUriRequested(const QUrl& url);
  void onVolumeChangedRequested(double volume);

  void updateMetadata();
};

}  // namespace netease

#endif  // DBUS_MPRIS_ADAPTER_H_