// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#include "native/browser.h"

#include <QDebug>
#include <QList>
#include <qcef_cookie_store.h>

#include "application.h"
#include "service/settings_manager.h"

namespace netease {

namespace {

const char kLocalStorageGroupName[] = "NativeBrowser";

}  // namespace

Browser::Browser(QObject* parent) : QObject(parent) {
  this->setObjectName("browser");
}

QString Browser::getCookie(const QString& url, const QString& name) {
  return QCefGetCookie(url, name);
}

QVariantMap Browser::getCookies(const QString& domain) {
  return QCefGetCookies(domain);
}

void Browser::moveCookie(const QString& old_domain,
                         const QString& new_domain,
                         const QString& name) {
  return QCefMoveCookie(old_domain, new_domain, name);
}

bool Browser::removeCookie(const QString& domain, const QString& name) {
  return QCefRemoveCookie(domain, name);
}

void Browser::setCookie(const QString& domain, const QString &name,
                        const QString& value) {
  return QCefSetCookie(domain, name, value);
}

QString Browser::getLocalStorage(const QString& domain) {
  const QVariant result = SettingsGetValue(kLocalStorageGroupName, domain);
  if (result.isValid()) {
    return result.toString();
  }
  return "null";
}

void Browser::setLocalStorage(const QString& domain, const QString& value) {
  SettingsSetValue(kLocalStorageGroupName, domain, value);
}

}  // namespace netease
