// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#ifndef NATIVE_BROWSER_H_
#define NATIVE_BROWSER_H_

#include <QObject>
#include <QVariant>
class QWebElement;

namespace netease {

// Operate QtWebkit directly to bypass its security restrictions.
// Methods in this class run in ui thread.
class Browser : public QObject {
  Q_OBJECT

 public:
  explicit Browser(QObject* parent);

 signals:
  // Browser navigation command.
  // Not used on Linux platform.
  void onappcommand(int cmd);

  // TODO(LiuLang): Emit this signal.
  void onbeforepopup();
  void onbeforerequest();

  // TODO(xushaohua): Add comments.
  void onescape();

 public slots:
  // Get browser cookie with |name| as cookie name and |domain| as cookie path.
  // This method is used to bypass browser security restriction.
  // Returns cookie value on success, or "" on error.
  // TODO(xushaohua): Throws exception if not found.
  QString getCookie(const QString& domain, const QString& name);

  // Get all cookies associated with |domain|.
  // Returns a map of (cookie-name, cookie-value) pair.
  QVariantMap getCookies(const QString& domain);

  // Movie cookie with |name| from domain |old_domain| to domain |new_domain|.
  // TODO(xushaohua): Throws exception if cookie does not exist.
  void moveCookie(const QString& old_domain,
                  const QString& new_domain,
                  const QString& name);

  // Remove cookie with |name| at |domain|.
  // Returns true if this cookie exists and is removed, else false.
  bool removeCookie(const QString& domain, const QString& name);

  // Set browser cookie at |domain| with |name| and cookie value |value|.
  // TODO(xushaohua): Throws exception on error.
  void setCookie(const QString& domain,
                 const QString& name,
                 const QString& value);

  // Set/Get localStorage.
  QString getLocalStorage(const QString& domain);
  void setLocalStorage(const QString& domain, const QString& value);
};

}  // namespace netease

#endif  // NATIVE_BROWSER_H_
