// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#ifndef NATIVE_DOWNLOAD_H_
#define NATIVE_DOWNLOAD_H_

#include <QObject>
#include <QVariant>
#include <QVariantMap>

namespace netease {

// Provides downloading web resources interface.
class Download : public QObject {
  Q_OBJECT

 public:
  explicit Download(QObject* parent);

 signals:
  // Emitted after calling cancel() slot.
  void oncancel(const QString& task_id);

  // Emitted after calling queryDownloadSchedule().
  void ondownloadeshecdule(const QVariantList& result);

  // Emitted when new data received in download-workers.
  void onprocess(const QString& task_id, const QVariantMap& percent);

 public slots:
  // Cancel downloading task, remove temporary files and emit oncancel().
  void cancel(const QString& task_id);

  // Pause downloading task.
  void pause(const QString& task_id);

  // Get download schedule and emit ondownloadschedule() signal.
  // |task_list| is a list of map containing:
  //  * id, task_id
  //  * path
  void queryDownloadShecdule(const QVariantList& task_list);

  // Add a new download task.
  // |task_info| is an object containing these fields:
  //   * id: task_id.
  //   * url: resource url currently available.
  //   * rel_path: filepath to save downloaded file to.
  //   * pre_path: filepath used at last time, might be empty. If pre_path is
  //   *   is not the same as rel_path, move that file to rel_path first.
  //   * type: 1, when saving images.
  void start(const QVariantMap& task_info);

 private:
  void initConnections();
};

}  // namespace netease

#endif  // NATIVE_DOWNLOAD_H_
