// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#include "native/music_library.h"

#include <QDebug>

#include "application.h"
#include "service/local_music_manager.h"
#include "service/media_metadata.h"

namespace netease {

namespace {

// Current music library version.
const char kCurrentLibraryVersion[] = "3";
const char kLastLibraryVersion[] = "3";

}  // namespace

MusicLibrary::MusicLibrary(QObject* parent) : QObject(parent) {
  this->setObjectName("musiclibrary");
  this->initConnections();
}

void MusicLibrary::addLibrary(const QString& library_path, int song_limit) {
  emit app->local_music_manager->addLibrary(library_path, song_limit);
}

void MusicLibrary::execSql(const QString& id, const QString& statement) {
  emit app->local_music_manager->execSql(id, statement);
}

void MusicLibrary::execSqls(const QString& id, const QStringList& statements) {
  emit app->local_music_manager->execTransaction(id, statements);
}

QString MusicLibrary::getLibraryPath(const QString& library_path) {
  return GetRealLibraryPath(library_path);
}

void MusicLibrary::initConnections() {
  connect(app->local_music_manager, &LocalMusicManager::onExecSqlDone,
          this, &MusicLibrary::onexecsql);
  connect(app->local_music_manager, &LocalMusicManager::onAddEnd,
          this, &MusicLibrary::onaddend);
  connect(app->local_music_manager, &LocalMusicManager::onAddProgress,
          this, &MusicLibrary::onaddprogress);
  connect(app->local_music_manager, &LocalMusicManager::onObserveLibrary,
          this, &MusicLibrary::onobserveLibrary);
  connect(app->local_music_manager, &LocalMusicManager::onRemoveLibrary,
          this, &MusicLibrary::onremovelibrary);
  connect(app->local_music_manager, &LocalMusicManager::onRemoveLibraryItems,
          this, &MusicLibrary::onremovelibraryitems);

  connect(app->media_metadata, &MediaMetadata::onReadAudioInfo,
          this, &MusicLibrary::onreadmusicinfo);
  connect(app->media_metadata, &MediaMetadata::onReadCueInfo,
          this, &MusicLibrary::onparsecueinfo);
}

void MusicLibrary::observeLibrary(const QString& library_path) {
  emit app->local_music_manager->observeLibrary(library_path);
}

void MusicLibrary::queryLibraryVersion() {
  emit onquerylibraryversion(kCurrentLibraryVersion, kLastLibraryVersion);
}

void MusicLibrary::parseCueInfo(const QString& id, const QString& filepath,
                                bool unused) {
  Q_UNUSED(unused);
  emit app->media_metadata->readCueInfo(id, filepath);
}

void MusicLibrary::readMusicInfo(const QString& id, const QString& filepath,
                                 bool unused) {
  qDebug() << Q_FUNC_INFO << id << filepath << unused;
  Q_UNUSED(unused);
  emit app->media_metadata->readAudioInfo(id, filepath);
}

void MusicLibrary::removeLibrary(const QString& library_path) {
  emit app->local_music_manager->removeLibrary(library_path);
}

void MusicLibrary::removeLibraryItems(const QVariantList& items) {
  emit app->local_music_manager->removeLibraryItems(items);
}

void MusicLibrary::removeObserveLibrary(const QString& library_path) {
  emit app->local_music_manager->removeObserveLibrary(library_path);
}

}  // namespace netease
