// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#include "native/os.h"

#include <QApplication>
#include <QCursor>
#include <QDebug>
#include <QDesktopServices>
#include <QDesktopWidget>
#include <QDir>
#include <QFile>
#include <QFileInfo>
#include <QJsonArray>
#include <QUrl>

#include "application.h"
#include "service/dbus_manager.h"
#include "service/resources_manager.h"

namespace netease {

namespace {

// TODO(xushaohua): Define OS version information.
const int kOsVersion = 4;

const char kStatusSuccess[] = "success";

}  // namespace

OS::OS(QObject* parent) : QObject(parent) {
  this->setObjectName("os");
}

void OS::checkNativeSupportFonts(const QStringList& fonts) {
  const QStringList existed_fonts = CheckNativeSupportFonts(fonts);
  emit this->onCheckNativeSupportFonts(kStatusSuccess, existed_fonts);
}

QString OS::getADDeviceID() {
  return GetAdDeviceId();
}

QString OS::getDeviceId() {
  return GetAdDeviceId();
}

QString OS::getDiskSpace(const QString& path) {
  return GetDiskSpace(path);
}

void OS::getKeyboardState() const {
  qDebug() << "TODO [OS]::getKeyboardState()";
}

int OS::getOsVer() const {
  return kOsVersion;
}

QVariantMap OS::getSystemInfo(const QString& type) const {
  if (type == "monitor") {
    QVariantMap result;

    QVariantMap work_area;

    QPoint point = QCursor::pos();
    const QRect geometry = QApplication::desktop()->screenGeometry(point);
    work_area.insert("x", QVariant(geometry.x()));
    work_area.insert("y", QVariant(geometry.y()));
    work_area.insert("width", QVariant(geometry.width()));
    work_area.insert("height", QVariant(geometry.height()));
    result.insert("workArea", work_area);
    return result;
  } else {
    return QVariantMap();
  }
}

bool OS::hasFullscreenWindow() const {
  // TODO(xushaohua): Check window status.
  return false;
}

bool OS::isAeroStyleEnabled() const {
  return false;
}

bool OS::isFileExist(const QString& filepath) const {
  QFile file(filepath);
  return file.exists();
}

void OS::isOnLine() {
  emit onisonline(app->dbus_manager->isOnline());
}

void OS::navigateExternal(const QString& url) const {
  QDesktopServices::openUrl(QUrl(url));
}

QString OS::queryOsVer() {
  return GetOsVersion();
}

void OS::querySystemFonts() {
  const QStringList fonts = QuerySystemFonts();
  emit this->onQuerySystemFonts(kStatusSuccess, fonts);
}

void OS::shellExplor(const QString& filepath) const {
  const QString dirname = QFileInfo(filepath).absoluteDir().absolutePath();
  QDesktopServices::openUrl(QUrl::fromLocalFile(dirname));
}

void OS::shellOpen(const QString& path) const {
  QDesktopServices::openUrl(QUrl::fromLocalFile(path));
}

}  // namespace netease
