// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#include "native/tray_icon.h"

#include <QDebug>
#include <QIcon>
#include <QUrl>

#include "application.h"
#include "channel/orca_reply.h"
#include "service/system_tray_icon.h"

namespace netease {

TrayIcon::TrayIcon(QObject* parent) : QObject(parent) {
  this->setObjectName("tray_icon");

  initConnections();
}

void TrayIcon::initConnections() {
  connect(app->system_tray_icon, &SystemTrayIcon::onClick,
          this, &TrayIcon::onclick);
  connect(app->system_tray_icon, &SystemTrayIcon::onRightClick,
          this, &TrayIcon::onrightclick);
}

void TrayIcon::install() const {
  app->system_tray_icon->setVisible(true);
}

void TrayIcon::popBalloon(const QVariantMap& options) const {
  qDebug() << "TODO [TrayIcon]::popupBalloon() " << options;
  // TODO(xushaohua): Calls notification dbus.
}

void TrayIcon::setIcon(const QString& uri) const {
  const QString filepath(OrcaReply::OrcaToQrc(QUrl(uri)));
  app->system_tray_icon->setIcon(QIcon(filepath));
}

void TrayIcon::setToolTip(const QString& tooltip) const {
  app->system_tray_icon->setToolTip(tooltip);
}

void TrayIcon::uninstall() const {
  app->system_tray_icon->setVisible(false);
}

bool TrayIcon::wasInstall() const {
  return app->system_tray_icon->isVisible();
}

}  // namespace netease
