// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#ifndef NATIVE_TRAY_ICON_H_
#define NATIVE_TRAY_ICON_H_

#include <QObject>
#include <QVariantMap>

namespace netease {

class TrayIcon : public QObject {
  Q_OBJECT

 public:
  explicit TrayIcon(QObject* parent);

 signals:
  // Emitted when tray icon is clicked.
  void onclick();

  // Emitted when tray icon is clicked with right mouse button.
  void onrightclick();

 public slots:
  // Set this system tray visible on desktop.
  void install() const;

  // Popup a desktop notification.
  // |options| is a map with these fields:
  //   * title, string, title of notification, default is empty.
  //   * text, string, content of notification, default is empty.
  //   * icon, string, uri to local icon file.
  //   * hasSound, bool, display notification with sound effect, default
  //     is false. Ignored on linux platform.
  //   * delayTime, number, life time of this message, ignored on linux
  //     platform.
  void popBalloon(const QVariantMap& options) const;

  // Set icon of program tray.
  void setIcon(const QString& uri) const;

  // Set tooltip text of tray icon.
  void setToolTip(const QString& tooltip) const;

  // Set this system tray invisible on desktop.
  void uninstall() const;

  // Check system tray icon visibility on desktop.
  bool wasInstall() const;

 private:
  void initConnections();
};

}  // namespace netease

#endif  // NATIVE_TRAY_ICON_H_
