// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#ifndef NATIVE_UPDATE_H_
#define NATIVE_UPDATE_H_

#include <QObject>
#include <QVariant>

namespace netease {

// Update is to used get application version and check for available updates.
// Note: APIs in Update is not implemented on linux platform.
class Update : public QObject {
  Q_OBJECT

 public:
  explicit Update(QObject* parent);

 signals:
  // TODO(xushaohua): Add comments.
  void onGetDownloadVersion();

  // TODO(xushaohua): Add comments.
  void onupdateprogress();

 public slots:
  // Get version of specific module, like "1.9.1.105579".
  // If |module| is "core", returns version of native side,
  // else returns version of ntpk.
  QString getVersion(const QString& module) const;

  // Get program version, returns an object:
  //  * build, build-id, like "105579".
  //  * version, like "1.9.1".
  QVariantMap getVisualVersion() const;

  // Download new version and emit onupdateprogress() signal.
  // |update_info| is an object containing:
  //   * filename: "orpheus_install.exe"
  //   * hash: "DBA40DA314B7704EC3605016CFF29945"
  //   * installtype: "install"
  //   * param: ""
  //   * url: "http://igame.163.com/download/pc/cloud-music.exe"
  // TODO(xushaohua): Call APIs in packagekit.
  void startUpdate(const QVariantList& update_info);
};

}  // namespace netease

#endif  // NATIVE_UPDATE_H_
