// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#include "service/auto_start.h"

#include <QDebug>
#include <QDir>
#include <QFile>
#include <QFileInfo>
#include <QSettings>

#include "base/consts.h"
#include "base/file_util.h"

namespace netease {

namespace {

// Group name of desktop entry in .desktop file.
const char kDesktopEntryGroupName[] = "Desktop Entry";

// Name of Exec=xxx record in desktop entry.
const char kDesktopExecName[] = "Exec";

// Qrc path to .desktop file.
const char kDesktopFileSrc[] = ":/misc/netease-cloud-music.desktop";

// Option name to be appended to startup paramater list.
const char kOrcaStartup[] = " --orca-startup=";

// Regexp to match kOrcaStartup option.
const char kOrcaStartupRegexp[] = " --orca-startup=[^ ]+";

// Relative directory name to $HOME/.config/autostart/ folder.
const char kUserAutoStartDir[] = "autostart";

QString GetAutoStartFilePath() {
  // Dir path to ~/.config/autostart/.
  QDir autostart_dir = ConcateDir(GetConfigDir(),
                                        kUserAutoStartDir);
  const QString filepath = autostart_dir.filePath(kAppDesktopFile);
  CreateParentDirs(filepath);
  return filepath;
}

}  // namespace

bool GetAutoStart() {
  return QFile::exists(GetAutoStartFilePath());
}

bool SetAutoStart(bool enable) {
  QString desktop_file(GetAutoStartFilePath());

  if (enable) {
    if (QFile::exists(desktop_file)) {
      if (!QFile::remove(desktop_file)) {
        qWarning() << "[AutoStart]::setAutoStart() failed to remove file: "
                   << desktop_file;
        return false;
      }
    }

    return QFile::copy(kDesktopFileSrc, desktop_file);

  } else {
    if (QFile::exists(desktop_file)) {
      return QFile::remove(desktop_file);
    } else {
      return true;
    }
  }
}

void SetAutoStartOption(const QString& param) {
  // Parse .desktop file
  QSettings settings(GetAutoStartFilePath(), QSettings::IniFormat);
  settings.beginGroup(kDesktopEntryGroupName);
  QString exec = settings.value(kDesktopExecName).toString();
  exec.remove(QRegExp(kOrcaStartupRegexp));
  exec.append(kOrcaStartup);

  // Note that |param| shall be encoded already.
  exec.append(param);

  settings.setValue(kDesktopExecName, exec);
  settings.endGroup();
}

}  // namespace netease
