// Copyright (c) 2015 LiuLang. All rights reserved.
// Use of this source is governed by General Public License that can be found
// in the LICENSE file.

#include "service/backend/aes.h"

#include <string.h>
#include <openssl/aes.h>
#include <QDebug>

namespace netease {

namespace {

// Use AES128.
const int kEncryptLength = 128;

// This key is used to decrypt old media files only.
const char kAESDeclinedKey[] = "#14ljk_!\\\\]&0U<'(";

const char kAESKey[] = "#14ljk_!\\]&0U<'(";

QString AES_Decrypt(const QString& cipher, bool with_base64) {
  std::string cipher_s = cipher.toStdString();
  QByteArray b;
  if (with_base64) {
    b = QByteArray::fromBase64(cipher.toLatin1());
  } else {
    b = QByteArray::fromHex(cipher.toLatin1());
  }
  const char* text = b.constData();

  AES_KEY aes_key;
  if (with_base64) {
    AES_set_decrypt_key((unsigned char*)kAESKey, kEncryptLength, &aes_key);
  } else {
    AES_set_decrypt_key((unsigned char*)kAESDeclinedKey, kEncryptLength,
                        &aes_key);
  }

  // Decrypt every block
  const int text_len = b.length();
  const int blk_num = (text_len / AES_BLOCK_SIZE);
  const int dec_len = text_len;
  uint8_t* dec_s = new uint8_t[dec_len];
  for (int i = 0; i < blk_num; ++i) {
    AES_ecb_encrypt((unsigned char*)((char*)text + i * AES_BLOCK_SIZE),
                    (unsigned char*)((char*)dec_s + i * AES_BLOCK_SIZE),
                    &aes_key, AES_DECRYPT);
    AES_decrypt((unsigned char*)((char*)text + i * AES_BLOCK_SIZE),
                    (unsigned char*)((char*)dec_s + i * AES_BLOCK_SIZE),
                    &aes_key);
  }

  // Remove paddings
  const unsigned char last_c = dec_s[dec_len - 1];
  int msg_size = dec_len - (int)last_c;
  if (dec_s[msg_size] != last_c) {
    msg_size = dec_len;
  }

  QByteArray b_result((char*)dec_s, msg_size);
  delete[] dec_s;
  return b_result;
}

}  // namespace

QString AES_Encrypt(const QString& msg) {
  const std::string msg_s = msg.toStdString();
  const char* text = msg_s.c_str();

  AES_KEY aes_key;
  AES_set_encrypt_key((unsigned char*)kAESKey, kEncryptLength, &aes_key);

  /* calc block num, should add a '\0' at last */
  const int text_len = strlen(text);
  const int blk_num = (text_len / AES_BLOCK_SIZE) + 1;

  // Copy text to align buffer and pad by PKCS5Padding
  // pad by algin_len - text_len
  // If text_len % AES_BLOCK_SIZE == 0, should add a
  // block at last and pad by 0x16
  // if diff 1, pad { ... , ...., 0x01 }
  // if diff 2, pad { ...., 0x02, 0x02 }
  // if diff 3, pad { 0x03, 0x03, 0x03 }
  const int alg_len = blk_num * AES_BLOCK_SIZE;
  uint8_t* alg_s =new uint8_t[alg_len];
  memcpy(alg_s, text, text_len);

  const int pad = AES_BLOCK_SIZE - text_len % AES_BLOCK_SIZE;
  for (int i = text_len; i < alg_len; i++) {
      alg_s[i] = pad;
  }

  // Encrypt every block
  uint8_t* enc_s = new uint8_t[alg_len];
  for (int i = 0; i < blk_num; i++) {
    AES_ecb_encrypt((unsigned char*)((char*)alg_s + i * AES_BLOCK_SIZE),
                    (unsigned char*)((char*)enc_s + i * AES_BLOCK_SIZE),
                    &aes_key, AES_ENCRYPT);
  }

  QByteArray b((char*)enc_s, alg_len);
  delete[] enc_s;
  return b.toBase64();
}

QString AES_Decrypt(const QString& cipher) {
  const QString tmp(AES_Decrypt(cipher, false));
  // If it is not valid json object, try fallback mode.
  if (!tmp.left(20).contains('{')) {
    return AES_Decrypt(cipher, true);
  }
  return tmp;
}

}  // namespace netease

