// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#include "service/backend/download_song_info.h"

#include <QDebug>
#include <QSettings>

#include "base/file_util.h"

namespace netease {

namespace {

const char kInfoFileExtension[] = ".info";

const char kSettingsMd5[] = "md5";
const char kSettingsTotalSize[] = "total-size";
const char kSettingsDownloadedSize[] = "downloaded-size";

}  // namespace

// Get info path path associated with file at |filepath|.
QString GetDownloadInfoFile(const QString& filepath) {
  return filepath + kInfoFileExtension;
}

DownloadSongInfo::DownloadSongInfo(QObject* parent)
    : QObject(parent),
      total_size(0),
      downloaded_size(0),
      md5(),
      info_filepath_() {
  this->setObjectName("download_song_info");
}

DownloadSongInfo::~DownloadSongInfo() {
}

bool DownloadSongInfo::exists() const {
  Q_ASSERT(info_filepath_.length() > 0);
  return QFile::exists(info_filepath_);
}

void DownloadSongInfo::read() {
  Q_ASSERT(info_filepath_.length() > 0);
  QSettings settings(info_filepath_, QSettings::IniFormat);
  settings.setValue(kSettingsMd5, md5);
  total_size = settings.value(kSettingsTotalSize).toLongLong();
  downloaded_size = settings.value(kSettingsDownloadedSize).toLongLong();
  md5 = settings.value(kSettingsMd5).toString();
}

void DownloadSongInfo::remove() {
  Q_ASSERT(info_filepath_.length() > 0);
  QFile::remove(info_filepath_);
}

void DownloadSongInfo::reset() {
  md5 = "";
  total_size = 0;
  downloaded_size = 0;
}

void DownloadSongInfo::setSongFilepath(const QString& filepath) {
  Q_ASSERT(filepath.length() > 0);

  // Reset old info.
  reset();
  info_filepath_ = GetDownloadInfoFile(filepath);
}

void DownloadSongInfo::write() {
  Q_ASSERT(info_filepath_.length() > 0);
  QSettings settings(info_filepath_, QSettings::IniFormat);
  settings.setValue(kSettingsMd5, md5);
  settings.setValue(kSettingsTotalSize, total_size);
  settings.setValue(kSettingsDownloadedSize, downloaded_size);
}

}  // namespace netease
