// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#ifndef SERVICE_BACKEND_DOWNLOAD_SONG_INFO_H_
#define SERVICE_BACKEND_DOWNLOAD_SONG_INFO_H_

#include <QObject>

// Manages download info file.
// File size is stored in this file.
// If file size does not match, remove old file.
// If file is downloaded, call DeleteDownloadInfoFile() to delete its
// associated info file.
namespace netease {

QString GetDownloadInfoFile(const QString& filepath);

class DownloadSongInfo : public QObject {
  Q_OBJECT

 public:
  explicit DownloadSongInfo(QObject* parent = nullptr);
  ~DownloadSongInfo() override;

  qint64 total_size;
  qint64 downloaded_size;
  QString md5;

  void remove();

  // Check song info is stored on the disk.
  bool exists() const;

  // Read song info from disk.
  // Call exists() before this.
  void read();

  // Reset old song info with new |filepath|.
  void setSongFilepath(const QString& filepath);

  // Write song info to disk.
  void write();

 private:
  void reset();

  QString info_filepath_;
};


}  // namespace netease

#endif  // SERVICE_BACKEND_DOWNLOAD_SONG_INFO_H_
