// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#ifndef SERVICE_BACKEND_FILESYSTEM_WATCHER_H_
#define SERVICE_BACKEND_FILESYSTEM_WATCHER_H_

#include <QObject>
#include <QDir>
#include <QVector>
class QSocketNotifier;

namespace netease {

class FilesystemWatcher : public QObject {
  Q_OBJECT

 public:
  explicit FilesystemWatcher(QObject* parent = nullptr);
  ~FilesystemWatcher() override;

  struct WatchItem {
    int watch_id = 0;
    QString root_dir;
    QString dir;
    bool recursive = false;
  };

 signals:
  void fileAdded(const QString& root_dir, const QString& filepath);
  void fileDeleted(const QString& root_dir, const QString& filepath);

  void folderAdded(const QString& root_dir, const QString& dir);
  void folderDeleted(const QString& root_dir, const QString& dir);

  // Add all of files in |root_dir| to watch list.
  // If |recursive| is true, also sub directories are added to list.
  // Emit onWatchFolder() signal.
  void watchFolder(const QString& root_dir, bool recursive);
  void onWatchFolder(const QString& root_dir, bool ok);

  // Remove |root_dir| from watch list.
  void rmWatchFolder(const QString& root_dir);

 private:
  bool initInotify();

  // Add sub folders to watch list, recursively.
  bool watchSubFolders(const QString& root_dir, const QString& parent_dir);

  // Global inotify file descriptor.
  int inotify_fd_;

  QSocketNotifier* socket_notifier_ = nullptr;

  QVector<WatchItem> watch_items_;

  void addWatchItem(int watch_id, const QString& root_dir,
                    const QString& dir, bool recursive);
  int getWatchItemById(int watch_id);
  void removeAllWatchItems();
  void removeWatchItemById(int watch_id);
  void removeWatchItemsByRootDir(const QString& root_dir);
  void removeWatchItemsByParentDir(const QString& parent_dir);
  void removeWatchItemByPath(const QString& path);

 private slots:
  void handleWatchFolder(const QString& root_dir, bool recursive);
  void handleRemoveWatchFolder(const QString& root_dir);
  void handleInotifyActivated();

  void doWatchFolder(const QString& root_dir,
                     const QString& dir,
                     bool recursive,
                     bool initial=false);
};

QDebug& operator<<(QDebug& dbug, const FilesystemWatcher::WatchItem& item);

}  // namespace netease

#endif  // SERVICE_BACKEND_FILESYSTEM_WATCHER_H_
