// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#include "service/backend/fontconfig_util.h"

#include <fontconfig/fontconfig.h>
#include <QDebug>

namespace netease {
namespace backend {

QStringList QueryFontsByLang(const QString& lang) {
  QStringList fonts;
  FcFontSet* font_set = nullptr;
  FcPattern* pattern = nullptr;
  FcObjectSet* object_set = nullptr;
  pattern = FcPatternCreate();
  object_set = FcObjectSetBuild(FC_LANG, FC_FAMILY, FC_FAMILYLANG, nullptr);

  if (!pattern || !object_set) {
    qCritical() << "[service]::QueryFontByLang() failed to create object set!";
    return fonts;
  }

  font_set = FcFontList(nullptr, pattern, object_set);

  if (!font_set) {
    qCritical() << "[service]::QueryFontByLang() failed to create font set!";
    return fonts;
  }

  if (object_set) {
    FcObjectSetDestroy(object_set);
  }
  if (pattern) {
    FcPatternDestroy(pattern);
  }

  for (int index = 0; index < font_set->nfont; ++index) {
    QString langs = (char*) FcPatternFormat(font_set->fonts[index],
                                            (FcChar8*)"%{lang}");
    if (langs.contains(lang)) {
      QString font_family = (char*) FcPatternFormat(font_set->fonts[index],
                                                    (FcChar8*)"%{family}");
      // familylang might be: "en,zh-tw,zh-cn".
      QString familylang = (char*) FcPatternFormat(font_set->fonts[index],
                                                   (FcChar8*)"%{familylang}");
      QStringList family_langs = familylang.split(QChar(','));
      QStringList font_families = font_family.split(QChar(','));
      const int lang_index = family_langs.indexOf(lang);
      if (lang_index > -1 && lang_index < font_families.size()) {
        const QString font_name = font_families.at(lang_index);
        if (fonts.indexOf(font_name) == -1) {
          fonts.append(font_name);
        }
      }
    }
  }

  FcFontSetDestroy(font_set);

  // FcCacheFini: Assertion `fcCacheChains[i] == ((void *)0)' failed.
//  FcFini();
  return fonts;
}

}  // namespace backend
}  // namespace netease