/*
 * Copyright (C) 2017 ~ 2017 Deepin Technology Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef NETEASE_CLOUD_MUSIC_SERVICE_BACKEND_LOCAL_MUSIC_SCANNER_H
#define NETEASE_CLOUD_MUSIC_SERVICE_BACKEND_LOCAL_MUSIC_SCANNER_H

#include <QObject>

namespace netease {

struct LocalMediaRecord {
  QString file;
  QString dir;
  QString title;
  QString artist;
  QString album;
  long duration;
  uint timestamp;
  int bitrate;
  qint64 file_size;
  QString id;
  QString parent_dir;
  QString library_path;
  QString source;  // Used in cue files
  int track_id;
};
typedef QList<LocalMediaRecord> LocalMediaRecordList;

class LocalMusicScanner : public QObject {
  Q_OBJECT

 public:
  explicit LocalMusicScanner(QObject* parent = nullptr);
  ~LocalMusicScanner() override;

 signals:
  void scan(const QString& library_path,
            const QString& root_dir,
            const QStringList& files);
  void onScan(const QString& library_path,
              const QString& root_dir,
              int task_count,
              const LocalMediaRecordList& media_list);

  void scanAll(const QString& root_dir, const QStringList& files);
  void onScanAll(const QString& root_dir,
                 const LocalMediaRecordList& media_list);

 private slots:
  void doScan(const QString& library_path,
              const QString& root_dir,
              const QStringList& files);
  void doScanAll(const QString& root_dir, const QStringList& files);
};

}  // namespace netease

#endif  // NETEASE_CLOUD_MUSIC_SERVICE_BACKEND_LOCAL_MUSIC_SCANNER_H
