/*
 * Copyright (C) 2017 ~ 2017 Deepin Technology Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef NETEASE_CLOUD_MUSIC_SERVICE_BACKEND_MEDIA_METADATA_READER_H
#define NETEASE_CLOUD_MUSIC_SERVICE_BACKEND_MEDIA_METADATA_READER_H

#include <QList>
#include <QString>

namespace netease {

QString EncryptMediaComment(const QString& comment);
QString DecryptMediaComment(QString comment);
bool IsValidMediaComment(const QString& comment);

struct AudioMetadata {
  QString title;
  QString album;
  QString artist;

  // Encrypted comment message is written to |comment| field so that it
  // can be used to identify media resource.
  QString comment;

  // Album cover image filepath. Used when updating metadata.
  QString cover_filepath;
  qint64 size = 0;
  long duration = 0;
  int bitrate = 0;
  int sample_rate = 0;
  int track_id = 0;
};

struct CueTrack {
  int track_id = 0;
  QString title;
  QString performer;
  long start = 0;
  long length = 0;  // frame length
  long duration = 0;
  qint64 size = 0;
  QString date;
};

struct CueMetadata {
  QString title;
  QString performer;
  QString genre;
  QString data_filename;
  QString cue_path;
  QString data_path;
  QString date;
  QList<CueTrack> tracks;
  qint64 size = 0; // size of data file.
  long duration = 0; // total duration.
  int bitrate = 0;
};

AudioMetadata ReadAudioInfo(const QString& filepath);
CueMetadata ReadCueInfo(const QString& filepath);

}  // namespace netease

#endif  // NETEASE_CLOUD_MUSIC_SERVICE_BACKEND_MEDIA_METADATA_READER_H
