// Copyright (c) 2015 Deepin. All rights reserved.
// Use of this source is governed by General Public License that can be found
// in the LICENSE file.

#ifndef SERVICE_BACKEND_SIMPLE_DOWNLOAD_MANAGER_H_
#define SERVICE_BACKEND_SIMPLE_DOWNLOAD_MANAGER_H_

#include <QObject>
#include <QList>

namespace netease {

class SimpleDownloadWorker;

// Wraps up simple download worker.
// Other classes, like AlbumCoverCache and MediaPlayerCache can inherit this
// class.
class SimpleDownloadManager : public QObject {
  Q_OBJECT

 public:
  explicit SimpleDownloadManager(QObject* parent = nullptr);
  ~SimpleDownloadManager() override;

 signals:
  // Emitted after download() finished;
  void onDownloaded(const QString& url, const QString& filepath);

  // Emitted after download() goes error.
  void onError(const QString& url, const QString& filepath);

  // Download album cover or song lyric with |url| and save to |filepath|.
  void download(const QString& url, const QString& filepath);

 protected:
  void initWorkers(int num_workers);

 private:
  void loadNextTask();

  // Reset worker state.
  // Return true when worker is found by |url|.
  bool resetWorkerByUrl(const QString& url, const QString& filepath);

  struct Task {
    QString url;
    QString filepath;
  };
  QList<Task> tasks_;

  struct Worker {
    SimpleDownloadWorker* worker;
    QString url;
    QString filepath;
    bool ready;
  };

  QList<Worker> workers_;

  // |workers_| shall be inited just once.
  bool workers_inited_;

 private slots:
  void handleWorkerDownloaded(const QString& url, const QString& filepath);
  void handleWorkerError(const QString& url, const QString& filepath);

  void handleDownload(const QString& url, const QString& filepath);
};

}  // namespace netease

#endif  // SERVICE_BACKEND_SIMPLE_DOWNLOAD_MANAGER_H_
