// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#ifndef SERVICE_BACKEND_SIMPLE_DOWNLOAD_WOKER_H_
#define SERVICE_BACKEND_SIMPLE_DOWNLOAD_WOKER_H_

#include <QObject>
#include <QNetworkReply>
class QFile;
class QNetworkAccessManager;
class QNetworkRequest;

namespace netease {

// Download small files, like album covers or song lyrics.
// Does not support resuming from last breakpoint.
class SimpleDownloadWorker : public QObject {
  Q_OBJECT

 public:
  explicit SimpleDownloadWorker(QObject* parent = nullptr);
  ~SimpleDownloadWorker();

 signals:
  // Emit this signal to start downloading.
  void download(const QString& url, const QString& filepath);

  // Emitted after file is downloaded.
  void onDownloaded(const QString& url, const QString& filepath);

  // Emitted when downloading process failed.
  void onError(const QString& url, const QString& filepath);

 private:
  void initConnections();
  void reset();
  void startDownload();

  // Filepath this file is stored at.
  QString filepath_;

  QString tmp_filepath_;

  // Resource url.
  QString url_;

  // Refers to QFile object with path |file_path_|.
  QFile* file_handler_ = nullptr;

  QNetworkAccessManager* network_manager_ = nullptr;
  QNetworkRequest* request_ = nullptr;
  QNetworkReply* reply_ = nullptr;

 private slots:
  // Handles QNetworkReply::error() signal.
  void handleError(QNetworkReply::NetworkError code);

  // Handles QNetworkReply::finished() signal.
  void handleFinished();

  // Handles QNetworkReply::readyRead() signal.
  void handleReadyRead();

  // Handles onStart() signal.
  void handleStartDownload(const QString& url, const QString& filepath);
};

}  // namespace netease

#endif  // SERVICE_BACKEND_SIMPLE_DOWNLOAD_WOKER_H_
