// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#include "service/backend/zip_wrapper.h"

#include <QDebug>
#include <QFile>
#include <QFileInfo>

#include "third_party/quazip/quazip/quazip.h"
#include "third_party/quazip/quazip/quazipfile.h"

namespace netease {
namespace backend {

bool Zip(const QString& dest_file, const QString& src_file) {
  QStringList file_list;
  file_list.append(src_file);
  return Zip(dest_file, file_list);
}

bool Zip(const QString& dest_file, const QStringList& file_list) {
  bool ok = true;
  if (QFile::exists(dest_file)) {
    ok = QFile::remove(dest_file);
    if (! ok) {
      qCritical() << "[base]::Zip() failed to remove existing zip file"
                  << dest_file;
      return ok;
    }
  }

  QuaZip zip_archive(dest_file);

  if (! zip_archive.open(QuaZip::mdCreate)) {
    qCritical() << "[base]::Zip() failed to open zip file"
               << dest_file;
    return false;
  }

  QuaZipFile zip_file(&zip_archive);
  for (const QString& filepath : file_list) {
    QFileInfo input_fileinfo(filepath);
    if (! input_fileinfo.exists()) {
      qCritical() << "[base]::Zip() file not exist!" << filepath;
      ok = false;
      break;
    }
    QFile input_file(filepath);
    if (! input_file.open(QIODevice::ReadOnly)) {
      qCritical() << "[base]::Zip() failed to open file: " << filepath;
      ok = false;
      break;
    }
    if (! zip_file.open(QIODevice::WriteOnly,
                        QuaZipNewInfo(input_fileinfo.fileName(),
                                      input_fileinfo.fileName()))) {
      qCritical() << "[base]::Zip() failed to open QuaZipFile(): " << dest_file;
      ok = false;
      input_file.close();
      break;
    }
    zip_file.write(input_file.readAll());

    if (zip_file.getZipError() != UNZ_OK) {
      qCritical() << "[base]::Zip() error occurs: " << zip_file.errorString();
      ok = false;
      break;
    }
    input_file.close();
    zip_file.close();
  }

  zip_archive.close();
  if (! ok) {
    QFile::remove(dest_file);
  }

  return ok;
}

}  // namespace backend
}  // namespace netease
