// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#include "service/command_line.h"

#include <QCommandLineOption>
#include <QCommandLineParser>
#include <QDebug>
#include <QUrl>

#include "base/consts.h"
#include "base/file_util.h"

namespace netease {

namespace {

// Command line option name used when migrating local files to new folder.
const char kArgsMoveRunValue[] = "moverun";

}  // namespace

QString GetMediaFileFromArgs() {
  QCommandLineParser parser;
  parser.addPositionalArgument("file", "Play a local media file.");
  parser.parse(QApplication::arguments());
  const QStringList files = parser.positionalArguments();
  // Only opens the first file.
  for (const QString& file : files) {
    if (IsMediaFile(file)) {
      // Argument might be URL or local file path.
      if (file.startsWith("file://")) {
        return QUrl(file).toLocalFile();
      } else {
        return file;
      }
    }
  }
  return "";
}

QString CommandLineToQuery() {
  QCommandLineParser parser;

  QCommandLineOption startup(kArgsOrpheusStartupName,
                             QObject::tr("Startup mode."),
                             kArgsOrpheusStartupName);
  parser.addOption(startup);

  QCommandLineOption require(kArgsRequireName,
                             QObject::tr("Is required or not"),
                             kArgsRequireName);
  parser.addOption(require);

  QCommandLineOption move_src(kArgsMoveRunSrcName,
                              QObject::tr("Old path to download folder"),
                              kArgsMoveRunSrcName);
  parser.addOption(move_src);

  QCommandLineOption move_dst(kArgsMoveRunDstName,
                              QObject::tr("New path to download folder"),
                              kArgsMoveRunDstName);
  parser.addOption(move_dst);

  QCommandLineOption web_argument(kArgsWebArgumentName,
                                  QObject::tr("Options used at webview"),
                                  kArgsWebArgumentName);
  parser.addOption(web_argument);

  parser.parse(QApplication::arguments());

  QString query;

  // Data migration.
  if (parser.isSet(kArgsOrpheusStartupName)
      && parser.value(kArgsOrpheusStartupName) == kArgsMoveRunValue) {
    query.append("action=migrate");
    query.append("&src=");
    query.append(
        parser.value(kArgsMoveRunSrcName).toUtf8().toPercentEncoding());
    query.append("&dest=");
    query.append(
        parser.value(kArgsMoveRunDstName).toUtf8().toPercentEncoding());
    query.append("&require=1");
    query.append("&");
    query.append(kArgsOrpheusStartupName);
    query.append("=");
    query.append(kArgsMoveRunValue);
  } else {
    // Construct options
    for (const QString& option_name : parser.optionNames()) {
      if (option_name.length() > 0 && parser.isSet(option_name)) {
        query.append(option_name);
        query.append("=");
        // Escape value.
        query.append(parser.value(option_name).toUtf8().toPercentEncoding());
        query.append("&");
      }
    }
  }

  // Remove spare ampersand mark.
  if (query.endsWith("&")) {
    query.remove(query.length() - 1, 1);
  }

  return query;
}

}  // namespace netease
