// Copyright (c) 2017 LiuLang. All rights reserved.
// Use of this source is governed by General Public License that can be found
// in the LICENSE file.

#include "service/custom_scheme_handler.h"

#include <QDebug>
#include <QUrlQuery>

#include "application.h"
#include "service/storage_manager.h"

namespace netease {

namespace {

const char kSkinLocal[] = "/default";
const char kSkinPrefixName[] = ":/webapp/style/skin%1";

}  // namespace

QString OrpheusSchemeHandler(const QUrl& url) {
  const QString scheme_name = url.scheme();
  const QString host = url.host();
  const QString path = url.path();
  if (scheme_name == "orpheus") {
    if (host == "orpheus") {
      if (path.startsWith("/storage/local")) {
        // Handles local storage.
        // Removes "file=" query name.
        const QString filename(url.query().mid(5));
        // Copy local file to temp file, so that its content can be update
        // on the fly.
        const QString temp_file = app->storage_manager->copyTempFile(filename);
        // Ignore temp file if it is empty.
        if (QFileInfo(temp_file).size() == 0) {
          return "";
        } else {
          return temp_file;
        }
      } else {
        return QString(":/webapp" + url.path());
      }
    } else if (host == "skin") {
      if (path.startsWith(kSkinLocal)) {
        // Remove skin path prefix.
        const int index = static_cast<int>(strlen(kSkinLocal));
        return QString(kSkinPrefixName).arg(path.mid(index));
      }
    } else if (host == "cache") {
      const QString filepath =
          app->storage_manager->getAlbumCover(url.toString());
      if (!filepath.isEmpty()) {
        return filepath;
      }

    } else if (host == "image") {
      // Handles local temp images.
      const QUrlQuery query(url.query(QUrl::FullyDecoded));
      const QStringList path_list = query.allQueryItemValues("path");
      if (path_list.length() != 1) {
        qWarning() << "[SchemeHandlerFactory]::Create() path is null";
        return "";
      }
      const QString path = path_list.at(0);

      const QStringList width_list = query.allQueryItemValues("width");
      if (width_list.length() != 1) {
        qWarning() << "[SchemeHandlerFactory]::Create() width is null";
        return "";
      }
      const int width = width_list.at(0).toInt();

      const QStringList height_list = query.allQueryItemValues("height");
      if (height_list.length() != 1) {
        qWarning() << __FUNCTION__ << "height is null";
        return "";
      }
      const int height = height_list.at(0).toInt();

      const QStringList scale_types = query.allQueryItemValues("scale_type");
      int scale_type;
      if (scale_types.length() != 1) {
        qWarning() << __FUNCTION__ << "scale type is null";
        scale_type = 1;
      } else {
        scale_type = scale_types.at(0).toInt();
      }
      return ScaleImage(path, width, height, scale_type);

    } else if (host == "upload") {
      // TODO(LiuLang): Support uploading files.
    }
  }

  return "";
}

}  // namespace netease