// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#ifndef SERVICE_DOWNLOAD_MANAGER_H_
#define SERVICE_DOWNLOAD_MANAGER_H_

#include <QObject>
#include <QList>
#include <QSet>
#include <QVariantMap>

namespace netease {

class DownloadWorker;

// Download music file,  lyric file or MV.
// Storage process is mostly handled in StorageManager class.
// This object works in background thread.
class DownloadManager : public QObject {
  Q_OBJECT

 public:
  explicit DownloadManager(QObject* parent = nullptr);
  ~DownloadManager();

 signals:
  // Emitted after |task_id| is canceled.
  void onCanceled(const QString& task_id);

  // Emitted when error occurs.
  void onError(const QString& task_id);

  // Emitted when new data received in download-workers.
  void onProcess(const QString& task_id, const QVariantMap& percent);

  // Emit this signal in to cancel a task.
  void cancel(const QString& task_id);

  // Emit this signal to pause a task.
  void pause(const QString& task_id);

  // Download web resources(mp3, mp4 or large images).
  // |task_info| is an object containing these fields:
  //   * id: task_id.
  //   * url: resource url currently available.
  //   * rel_path: filepath to save downloaded file to.
  //   * pre_path: filepath used at last time, might be empty. If pre_path is
  //   *   is not the same as rel_path, move that file to rel_path first.
  //   * md5:
  void download(const QVariantMap& task_info);

  void queryDownloadSchedule(const QVariantList& task_list);

  // |tasks_info| is a list of map containing:
  //   * id, task_id
  //   * shecdule, downloading percentage. 0.0~1.0
  void onQueryDownloadSchedule(const QVariantList& tasks_info);

 private:
  void initConnections();

  QList<DownloadWorker*> workers_;

 private slots:
  void handleDownload(const QVariantMap& task_info);
  void handleQueryDownloadSchedule(const QVariantList& task_list);
};

}  // namespace netease

#endif  // SERVICE_DOWNLOAD_MANAGER_H_
