// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#ifndef SERVICE_FEEDBACK_MANAGER_H_
#define SERVICE_FEEDBACK_MANAGER_H_

#include <QObject>
#include <QVariant>
class QNetworkAccessManager;
class QHttpMultiPart;

namespace netease {

// Send user feedback, log file and statistics file to server.
// This object is running in background thread.
class FeedbackManager : public QObject {
  Q_OBJECT

 public:
  explicit FeedbackManager(QObject* parent = nullptr);
  ~FeedbackManager() override;

 signals:
  // Post feedback message and log file to server.
  // Log file is compressed with zlib.
  void sendFeedback(const QVariantMap& info);

  // Post stat file to server.
  // server url is in UrlManager.
  void sendStatFile(const QString& cookie);

  // Emitted after calling sendStatFile() successfully.
  void onSendStatFile();

 private:
  void initConnections();

  QNetworkAccessManager* network_manager_ = nullptr;
  QHttpMultiPart* multi_part_ = nullptr;

 private slots:
  void handleSendFeedback(const QVariantMap& info);
  void handleSendStatFile(const QString& cookie);
};

}  // namespace netease

#endif  // SERVICE_FEEDBACK_MANAGER_H_
