// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#ifndef SERVICE_KEYBINDER_H_
#define SERVICE_KEYBINDER_H_

#include <QHash>
#include <QVariant>
#include <QWidget>
class QShortcut;

namespace netease {

class GlobalShortcut;

// Keybinder manages local and global keyboard shortcut.
class Keybinder : public QObject {
  Q_OBJECT

 public:
  explicit Keybinder(QObject* parent = nullptr);
  ~Keybinder() override;

 signals:
  // Emitted when a registered hotkey is triggered.
  void onHotkey(const QString& action_name, bool is_global);

  // Emitted after calling registerHotkey().
  void onRegisterHotkeyResult(const QString& action_name, bool is_global,
                              int error_code);

  // Emitted after calling unregisterHotkey().
  void onUnregisterHotkeyResult(const QString& action_name, bool is_global,
                                int error_code);

  // Register keyboard shortcut and emit onRegisterHotkeyResult() signal.
  // |action_name| is the action to be trigger when that key is pressed, like
  //   "play_pause".
  // |keys| is keycodes of keybinder group, like [17, 80] => "Ctrl+P".
  // If |is_global| is true, register it as global keybinding.
  void registerHotkey(const QString& action_name, const QVariantList& keys,
                      bool is_global);

  // Unregister keyboard shortcut and emit onUnergisterHotkeyResult() signal.
  void unregisterHotkey(const QString& action_name, bool is_global);


 public slots:
  // Handles QShortcut::activated() signal.
  void handleShortcutActivated();

 private:
  void initConnections();
  void registerGlobalHotkey(const QString& action_name,
                            const QVariantList& keys);
  void registerLocalHotkey(const QString& action_name, const QVariantList& keys);
  void unregisterGlobalHotkey(const QString& action_name);
  void unregisterLocalHotkey(const QString& action_name);

  QHash<QString, GlobalShortcut*> global_shortcut_;
  QHash<QString, QShortcut*> local_shortcut_;

 private slots:
  void handleRegisterHotkey(const QString& action_name,
                            const QVariantList& keys, bool is_global);

  void handleUnregisterHotkey(const QString& action_name, bool is_global);
};

}  // namespace netease

#endif // SERVICE_KEYBINDER_H_
