// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#ifndef SERVICE_LOG_MANAGER_H_
#define SERVICE_LOG_MANAGER_H_

#include <QObject>
class QFile;
class QTextStream;

namespace netease {

// LogManager manages web statistics, web log and native log.
// In debug mode, native log is also printed to console.
class LogManager : public QObject {
  Q_OBJECT

 public:
  explicit LogManager(QObject* parent = nullptr);
  ~LogManager() override;

  // Truncate log files.
  void resetLogFile();
  void resetStatFile();

  // Get temp log file, file already truncated to appropriate size.
  QString getTmpLogFilepath();
  QString getTmpStatFilepath();

 signals:
  // Write |msg| to log file.
  // |msg| is written to start of a new line.
  void log(const QString& msg);

  // Write |msg| to statis file.
  void statMsg(const QString& msg);

 private:
  // Initialize log manager.
  void init();

  void initConnections();

  QString log_filepath_;
  QString tmp_log_filepath_;
  QString native_log_filepath_;
  QString statis_filepath_;
  QString tmp_stat_filepath_;
  QFile* log_file_;
  QFile* statis_file_;

 private slots:
  void handleLog(const QString& msg);

  void handleStatMsg(const QString& msg);
};

}  // namespace netease

#endif  // SERVICE_LOG_MANAGER_H_
