// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by General Public License that can be found
// in the LICENSE file.

#ifndef SERVICES_MEDIA_METADATA_H_
#define SERVICES_MEDIA_METADATA_H_

#include <QObject>
#include <QList>
#include <QVariantMap>

#include "service/backend/media_metadata_reader.h"

namespace netease {

// Read/write metadata of media resource file.
class MediaMetadata : public QObject {
  Q_OBJECT

 public:
  explicit MediaMetadata(QObject* parent = nullptr);
  ~MediaMetadata() override;

 signals:
  void onAddId3(const QString& task_id, int success);
  void onReadCueInfo(const QString& id, int err, const QVariantMap& info,
                     const QString& reason);
  void onReadAudioInfo(const QString& id, const QString& filepath,
                       int err, const QVariantMap& info);

  // |filepath| is path to cue file.
  void readCueInfo(const QString& id, const QString& filepath);

  // |filepath| is path to audio file.
  void readAudioInfo(const QString& id, const QString& filepath);

  void addId3(const QString& task_id, const QString& media_path,
              const QString& album_cover_path, const QString& media_info,
              const QVariantMap& tag);

 private:
  void initConnections();
  bool writeFlacTag(const QString& media_path, const AudioMetadata& metadata);
  bool writeMp3Tag(const QString& media_path, const AudioMetadata& metadata);
  bool writeMp4Tag(const QString& media_path, const AudioMetadata& metadata);

 private slots:
  void handleAddId3(const QString& task_id, const QString& media_path,
                    const QString& album_cover_path, const QString& media_info,
                    const QVariantMap& tag);
  void handleReadCueInfo(const QString& id, const QString& filepath);

  void handleReadAudioInfo(const QString& id, const QString& filepath);
};

}  // namespace netease

#endif  // SERVICES_MEDIA_METADATA_H_