// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#ifndef SERVICE_MEDIA_PLAYER_CACHE_H_
#define SERVICE_MEDIA_PLAYER_CACHE_H_

#include "service/backend/simple_download_manager.h"
#include <QDir>

namespace netease {

// Manages local cache of media resource.
class MediaPlayerCache : public SimpleDownloadManager {
  Q_OBJECT

 public:
  explicit MediaPlayerCache(QObject* parent = nullptr);
  ~MediaPlayerCache() override;

  // Deleted cached songs.
  // Thread-safe.
  void clearCachedSongs();

  // Returns filepath to cached media resource.
  // If that resource is not cached, returns an empty string.
  // Thread-safe.
  QString getCachedPath(const QString& song_id, const QString& bitrate,
                        const QString& md5) const;

  // Check whether song with |song_id| and |bitrate| is cached.
  // Thread-safe.
  bool isCached(const QString& song_id, const QString& bitrate,
                const QString& md5) const;

 signals:
  // Load a new media resource. If that resource is not cached yet, download
  // it and returns an empty filepath.
  // |url| is the link of that resource.
  // Returns a filepath to local file.
  void load(const QString& url, const QString& song_id, const QString& bitrate,
            const QString& md5, const QString& format);

 private:
  void initConnections();

  QDir base_dir_;

  // Cache song is keep at this path before it is finished.
  QString temp_filepath_;

 private slots:
  // Handles DownloadManager::onCacheSongDownloaded() signal.
  void handleCacheSongDownloaded(const QString& url, const QString& filepath);

  void handleLoad(const QString& url, const QString& song_id,
                  const QString& bitrate, const QString& md5,
                  const QString& format);
};

}  // namespace netease

#endif  // SERVICE_MEDIA_PLAYER_CACHE_H_
