// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#ifndef SERVICE_POPUP_MENU_MANAGER_H_
#define SERVICE_POPUP_MENU_MANAGER_H_

#include <QObject>
#include <QVariant>
class QAction;
class QMenu;
class QTimer;

namespace netease {

class PopupMenu;

// Construct and display a popup menu at specific position.
class PopupMenuManager : public QObject {
  Q_OBJECT

 public:
  explicit PopupMenuManager(QObject* parent = nullptr);
  ~PopupMenuManager() override;

 signals:
  // Emitted when
  void onMenuHide();

  // Emitted when a menu item is clicked.
  void onMenuItemClicked(const QString& menu_id);

  void hidePopupMenu();

  // Popup context menu.
  // |conf| is an object containing:
  //   * content: a list of menu item. A menu item contains:
  //       * "text": "xx",
  //       * "menu":true,
  //       * "menu_id":10000,
  //       * "enable":true,
  //       * "separator":false,
  //       * "children":null,
  //       * "image_path": "orpheus://orpheus/style/res/menu/icn_lookfor.png"
  //   * hotkey: a map containing menu-item => hotkey pair.
  //   * menu_type, default value is "normal".
  // |menu_type| might be:
  //   * 0, main window.
  //   * 1, mini player.
  //   * 2, system tray.
  //   * 3, desktop lyrics.
  void popupMenu(const QVariantMap& conf, int menu_type);

 private:
  void initConnections();
  QMenu* menu_;

 private slots:
  void handleHidePopupMenu();
  void handleMenuActionTriggered(QAction* action);
  void handlePopupMenu(const QVariantMap& conf, int menu_type);
};

}  // namespace netease

#endif  // SERVICE_POPUP_MENU_MANAGER_H_
