// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#include "service/process_manager.h"

#include <sys/types.h>
#include <unistd.h>
#include <QDebug>
#include <QProcess>

#include "application.h"
#include "base/consts.h"
#include "base/process_util.h"
#include "service/dbus_manager.h"
#include "ui/window_manager.h"

namespace netease {

namespace {

// To store restart argument.
QStringList g_args;

// Sleep 3s before calling kill(9).
const int kForceExitSleep = 3;

bool g_restarted = false;

}  // namespace

void DoRestart() {
  qDebug() << Q_FUNC_INFO << g_args;

  if (g_restarted) {
    return;
  }
  g_restarted = true;

  if (!g_args.isEmpty()) {
    const QString& program = g_args.at(0);
    const QStringList args = g_args;
    if (!QProcess::startDetached(program, args)) {
      qCritical()
          << "[ProcessManager]::restart() failed to start child process:"
          << program << args;
    }

    // Kill renderer processes and self.
    // Does not work any more when application is restarted.
//    kill(- getpid(), SIGTERM);
    KillAllProcesses();
  }
}

ProcessManager::ProcessManager(QObject* parent)
    : QObject(parent),
      is_quited_(false) {
  this->setObjectName("process_manager");
}

ProcessManager::~ProcessManager() {

}

void ProcessManager::exit() {
  if (is_quited_) {
    return;
  }
  is_quited_ = true;

  app->quitApplication();
}

void ProcessManager::restart(const QStringList& arguments) {
  qDebug() << Q_FUNC_INFO << arguments;

  // Stop dbus service first.
  app->dbus_manager->unRegister();

  const QStringList current_arguments(app->arguments());
  QStringList modified_arguments(arguments);
  const QString program = current_arguments.at(0);
  modified_arguments.insert(0, program);

  for (int i = 1; i < current_arguments.length(); ++i) {
    const QString argument = current_arguments.at(i);
    // Ignores duplicated arguments.
    if (argument != kArgsMoveRun &&
        argument != kArgsInstall &&
        argument != kArgsQuitRun &&
        argument != kArgsRestart &&
        (! argument.startsWith(kArgsMoveRunSrc)) &&
        (! argument.startsWith(kArgsMoveRunDst)) &&
        (! argument.startsWith(kArgsRequire)) &&
        (! modified_arguments.contains(argument))) {
      modified_arguments.append(argument);
    }
  }
  g_args = modified_arguments;
  qDebug() << "g_args:" << g_args;

  // Kill self.
  this->exit();
}

}  // namespace netease
