// Copyright (c) 2015 Deepin. All rights reserved.
// Use of this source is governed by General Public License that can be found
// in the LICENSE file.

#include "service/resources_manager.h"

#include <QDebug>

#include "base/device_util.h"
#include "base/file_util.h"
#include "service/backend/fontconfig_util.h"
#include "service/settings_manager.h"

namespace netease {

namespace {

// Group name in settings.
const char kSettingsGroupName[] = "NativeOS";

// DeviceId key in settings.
const char kSettingsDeviceIdKey[] = "device-id";

}  // namespace

QString GetDiskSpace(const QString& path) {
  qint64 total_size = 0;
  qint64 freespace = 0;
  QString cache_path(path);
  if (path.isEmpty()) {
    cache_path = GetUserCacheDir().absolutePath();
  }
  if (!GetDiskUsage(cache_path, &total_size, &freespace)) {
    qWarning() << "GetDiskSpace failed to call statvfs:" << path;
  }

  QString info("{\"total\": %1, \"free\": %2}");
  return info.arg(total_size).arg(freespace);
}

QString GetAdDeviceId() {
  QString id = SettingsGetValue(kSettingsGroupName,
                                kSettingsDeviceIdKey).toString();
  if (id.isEmpty()) {
    id = GetDeviceId();
    SettingsSetValue(kSettingsGroupName, kSettingsDeviceIdKey, id);
  }
  return id;
}

QString GetOsVersion() {
  return GetDistroNameAndVersion();
}

QStringList CheckNativeSupportFonts(const QStringList& src_fonts) {
  QStringList zh_fonts = QuerySystemFonts();

  QStringList existed_fonts;
  for (const QString& font : src_fonts) {
    if (zh_fonts.indexOf(font) > -1) {
      existed_fonts.append(font);
    }
  }

  return existed_fonts;
}

QStringList QuerySystemFonts() {
  return backend::QueryFontsByLang("zh-cn");
}

}  // namespace netease