// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#include "service/settings_manager.h"

#include <QDebug>
#include <QSettings>

#include "base/consts.h"
#include "base/file_util.h"

namespace netease {

namespace {

// Path to global default settings.
const char kDefaultSettings[] = ":/misc/default-settings.ini";

}  // namespace

QVariant SettingsGetDefaultValue(const QString &group_name,
                                 const QString &key) {
  QSettings default_settings(kDefaultSettings, QSettings::IniFormat);
  default_settings.beginGroup(group_name);
  const QVariant result = default_settings.value(key);
  default_settings.endGroup();
  return result;
}

QVariant SettingsGetValue(const QString& group_name, const QString& key) {
  const QString path(GetUserConfigDir().filePath(kAppSettingsFile));
  QSettings settings(path, QSettings::IniFormat);
  settings.beginGroup(group_name);
  if (settings.contains(key)) {
    return settings.value(key);
  } else {
    return SettingsGetDefaultValue(group_name, key);
  }
}

void SettingsSetValue(const QString& group_name,
                      const QString& key,
                      const QVariant& value) {
  const QString path(GetUserConfigDir().filePath(kAppSettingsFile));
  QSettings settings(path, QSettings::IniFormat);
  settings.beginGroup(group_name);
  settings.setValue(key, value);
  settings.endGroup();
}

}  // namespace netease
