// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#ifndef SERVICE_SKIN_MANAGER_H_
#define SERVICE_SKIN_MANAGER_H_

#include <QColor>
#include <QObject>

namespace netease {

class SkinManager : public QObject {
  Q_OBJECT

 public:
  explicit SkinManager(QObject* parent = nullptr);
  ~SkinManager() override;

  struct Skin {
    // Name of skin folder, like "pub", "gold".
    QString name;

    // Content of skin info file. That file exists at root of each skin packet.
    QString info;
  };

  struct TextColors {
    QColor menu_item;
    QColor menu_disabled_item;

    // Lrc text color in MiniPlayer.
    QColor lrc_text;

    // Shadow color of lrc text in DesktopLyrics.
    QColor lrc_primary_shadow;
    QColor lrc_secondary_shadow;

    // Color of title and artist text in MiniPlayer.
    QColor no_lrc_title;
    QColor no_lrc_artist;

    // Color of title and artist in MiniPlayerInfoPanel.
    QColor info_panel_artist;
    QColor info_panel_title;

    // Colors in MiniPlayerPlaylist.
    QColor playlist_text;
    QColor playlist_hot;
    QColor playlist_selected;
  };

  TextColors text_colors;

  // Replace skin path "SKIN_ROOT" with current used skin path.
  QString absoluteSkinPath(const QString& path);

  // Returns button color.
  // Call QColor::isValid() to check its validity.
  const QColor buttonColor() const { return button_color_; }

  // Change current skin.
  // Change current skin, including color of buttons in mini player.
  // Returns true if that skin is valid.
  bool changeSkin(const QString& skin_type, const QString& skin_name,
                  const QColor& button_color);

  // Get name of skin currently used.
  QString currentSkin();

  QString formatStyle(const QString& content);

  // Convert "SKIN_ROOT" in style file with path |filepath| to appropriate
  // qrc path.
  QString formatStyleFile(const QString& filepath);

  // Get a list of all available local skins.
  QList<Skin> getLocalSkins();

  // Only "red" theme packet can be used to customize button colors.
  bool isCustomizedSkin() const;

 signals:
  // Emitted when choosing a new skin packet.
  void onUpdated(const QString& new_skin_name);

 private:
  void updateTextColors();

  QString current_skin_;

  // Color used in mini player buttons.
  QColor button_color_;
};

}  // namespace netease

#endif  // SERVICE_SKIN_MANAGER_H_
