// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#include "service/system_tray_icon.h"

#include <QMenu>
#include <QTimer>

#include "application.h"
#include "base/device_util.h"
#include "service/signal_manager.h"

namespace netease {

SystemTrayIcon::SystemTrayIcon(QObject* parent)
    : QSystemTrayIcon(parent),
      context_menu_(new QMenu()) {
  this->setObjectName("system_tray_icon");

  // Create context menu.
  // Menu content will be updated in PopupMenuManager.
  if (IsUbuntuUnity()) {
    this->setContextMenu(context_menu_);
  }

  this->initConnections();
}

SystemTrayIcon::~SystemTrayIcon() {
  delete context_menu_;
  context_menu_ = nullptr;
}

void SystemTrayIcon::initConnections() {
  connect(this, &SystemTrayIcon::activated,
          this, &SystemTrayIcon::handleTrayActivated);
  if (IsUbuntuUnity()) {
    connect(app->signal_manager, &SignalManager::onUpdateSystemTrayContextMenu,
            this, &SystemTrayIcon::onRightClick);
  }
}

void SystemTrayIcon::handleTrayActivated(ActivationReason reason) {
  switch (reason) {
  case ActivationReason::Trigger:
    emit onClick();
    break;
  case ActivationReason::Context:
    emit onRightClick();
    break;
  default:
    // Ignores other reasons.
    break;
  }
}

}  // namespace netease
