// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#include "service/thread_manager.h"

#include <QThread>

namespace netease {

namespace {

// Wait 2secs before terminating backgrounds threads.
const int kWaitForThreadTermination = 2;

}  // namespace

ThreadManager::ThreadManager(QObject* parent) : QObject(parent) {
  setObjectName("thread_manager");
  album_cover_cache_thread = new QThread();
  album_cover_cache_thread->start(QThread::NormalPriority);

  download_manager_thread = new QThread();
  download_manager_thread->start(QThread::NormalPriority);

  fs_watcher_thread = new QThread();
  fs_watcher_thread->start(QThread::LowPriority);

  media_player_cache_thread = new QThread();
  media_player_cache_thread->start(QThread::LowPriority);

  resources_manager_thread = new QThread();
  resources_manager_thread->start(QThread::LowestPriority);

  storage_thread = new QThread();
  storage_thread->start(QThread::NormalPriority);

  local_music_thread = new QThread();
  local_music_thread->start(QThread::NormalPriority);
}

ThreadManager::~ThreadManager() {
  if (album_cover_cache_thread->isRunning()) {
    album_cover_cache_thread->quit();
    album_cover_cache_thread->wait(kWaitForThreadTermination);
    album_cover_cache_thread->deleteLater();
  }

  if (download_manager_thread->isRunning()) {
    download_manager_thread->quit();
    download_manager_thread->wait(kWaitForThreadTermination);
    download_manager_thread->deleteLater();
  }

  if (fs_watcher_thread->isRunning()) {
    fs_watcher_thread->quit();
    fs_watcher_thread->wait(kWaitForThreadTermination);
    fs_watcher_thread->deleteLater();
  }

  if (media_player_cache_thread->isRunning()) {
    media_player_cache_thread->quit();
    media_player_cache_thread->wait(kWaitForThreadTermination);
    media_player_cache_thread->deleteLater();
  }

  if (resources_manager_thread->isRunning()) {
    resources_manager_thread->quit();
    resources_manager_thread->wait(kWaitForThreadTermination);
    resources_manager_thread->deleteLater();
  }

  if (storage_thread->isRunning()) {
    storage_thread->quit();
    storage_thread->wait(kWaitForThreadTermination);
    storage_thread->deleteLater();
  }

  if (local_music_thread->isRunning()) {
    local_music_thread->quit();
    local_music_thread->wait(kWaitForThreadTermination);
    local_music_thread->deleteLater();
  }
}

}  // namespace netease