/*
 * Copyright (C) 2017 ~ 2017 Deepin Technology Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "tests/frame_less_window.h"

#include <QDebug>
#include <QMouseEvent>

#include "ui/utils/xutil.h"
#include "move_resize_event_filter.h"

namespace netease {

FrameLessWindow::FrameLessWindow(QWidget* parent)
    : QFrame(parent),
      event_filter_(new MoveResizeEventFilter()) {
  this->setWindowFlags(Qt::FramelessWindowHint | Qt::Window);
//  event_filter_->startAction(this, MoveResizeEventFilter::Action::Move);
}

FrameLessWindow::~FrameLessWindow() {
  if (event_filter_ != nullptr) {
    delete event_filter_;
    event_filter_ = nullptr;
  }
}

void FrameLessWindow::mousePressEvent(QMouseEvent* event) {
  QFrame::mousePressEvent(event);
  qDebug() << Q_FUNC_INFO << event;

  const QPoint bottom_right = mapToGlobal(QPoint(width(), height()));
  const QPoint global_pos = event->globalPos();
  const QPoint delta = bottom_right - global_pos;
  const int kBorderSize = 5;
  if (delta.x() < kBorderSize && delta.y() < kBorderSize) {
    // bottom right corner.
//    utils::ResizeWindow(this, "bottomright");
  } else if (delta.x() < kBorderSize) {
    // right edge.
//    utils::ResizeWindow(this, "right");
  } else if (delta.y() < kBorderSize) {
    // bottom edge.
//    utils::ResizeWindow(this, "");
  } else {
    utils::MoveWindow(this);
  }
}

void FrameLessWindow::mouseMoveEvent(QMouseEvent* event) {
  QFrame::mouseMoveEvent(event);
}

}  // namespace netease
