/*
 * Copyright (C) 2017 ~ 2017 Deepin Technology Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "move_resize_event_filter.h"

#include <QAbstractEventDispatcher>
#include <QDebug>
#include <QWidget>
#include <QX11Info>
#include <xcb/xcb.h>
#include <X11/Xlib.h>

namespace {

//void MoveWindow(long window_id, int x, int y) {
//  auto display = QX11Info::display();
//  XMoveWindow(display, static_cast<Window>(window_id), x, y);
//  XFlush(display);
//}

void
print_modifiers(uint32_t mask) {
  const char** mod, * mods[] = {
      "Shift", "Lock", "Ctrl", "Alt",
      "Mod2", "Mod3", "Mod4", "Mod5",
      "Button1", "Button2", "Button3", "Button4", "Button5"
  };
  printf("Modifier mask: ");
  for (mod = mods; mask; mask >>= 1, mod++)
    if (mask & 1)
      //printf(*mod);
  putchar('\n');
}


}  // namespace

MoveResizeEventFilter::MoveResizeEventFilter() {

}

bool MoveResizeEventFilter::nativeEventFilter(const QByteArray& eventType,
                                              void* message,
                                              long* result) {
  if (eventType == "xcb_generic_event_t") {
    xcb_generic_event_t* event = static_cast<xcb_generic_event_t*>(message);
    const uint opcode = event->response_type & ~0x80;
    qDebug() << "type:" << event->response_type << opcode;

    switch (event->response_type & ~0x80) {
      case XCB_EXPOSE: {
        xcb_expose_event_t* ev = (xcb_expose_event_t*)event;

        printf(
            "Window %ld exposed. Region to be redrawn at location (%d,%d), with dimension (%d,%d)\n",
            (long)ev->window, ev->x, ev->y, ev->width, ev->height);
        break;
      }
      case XCB_BUTTON_PRESS: {
        xcb_button_press_event_t* ev = (xcb_button_press_event_t*)event;
        print_modifiers(ev->state);

        switch (ev->detail) {
          case 4:
            printf("Wheel Button up in window %ld, at coordinates (%d,%d)\n",
                   (long)ev->event, ev->event_x, ev->event_y);
            break;
          case 5:
            printf("Wheel Button down in window %ld, at coordinates (%d,%d)\n",
                   (long)ev->event, ev->event_x, ev->event_y);
            break;
          default:
            printf("Button %d pressed in window %ld, at coordinates (%d,%d)\n",
                   ev->detail, (long)ev->event, ev->event_x, ev->event_y);
        }
        break;
      }
      case XCB_BUTTON_RELEASE: {
        xcb_button_release_event_t* ev = (xcb_button_release_event_t*)event;
        print_modifiers(ev->state);

        printf("Button %d released in window %ld, at coordinates (%d,%d)\n",
               ev->detail, (long)ev->event, ev->event_x, ev->event_y);
        break;
      }
      case XCB_MOTION_NOTIFY: {
        xcb_motion_notify_event_t* ev = (xcb_motion_notify_event_t*)event;

        printf("Mouse moved in window %ld, at coordinates (%d,%d)\n",
               (long)ev->event, ev->event_x, ev->event_y);
        break;
      }
      case XCB_ENTER_NOTIFY: {
        xcb_enter_notify_event_t* ev = (xcb_enter_notify_event_t*)event;

        printf("Mouse entered window %ld, at coordinates (%d,%d)\n",
               (long)ev->event, ev->event_x, ev->event_y);
        break;
      }
      case XCB_LEAVE_NOTIFY: {
        xcb_leave_notify_event_t* ev = (xcb_leave_notify_event_t*)event;

        printf("Mouse left window %ld, at coordinates (%d,%d)\n",
               (long)ev->event, ev->event_x, ev->event_y);
        break;
      }
      case XCB_KEY_PRESS: {
        xcb_key_press_event_t* ev = (xcb_key_press_event_t*)event;
        print_modifiers(ev->state);

        printf("Key pressed in window %ld\n",
               (long)ev->event);
        break;
      }
      case XCB_KEY_RELEASE: {
        xcb_key_release_event_t* ev = (xcb_key_release_event_t*)event;
        print_modifiers(ev->state);

        printf("Key released in window %ld\n",
               (long)ev->event);
        break;
      }
      default:
        /* Unknown event type, ignore it */
        printf("Unknown event: %d\n", event->response_type);
        break;
    }

//    switch (opcode) {
//      case XCB_KEY_PRESS: {
//        qDebug() << "KEY press";
//        break;
//      }
//      case XCB_BUTTON_PRESS: {
//        qDebug() << "PRESS";
//        xcb_button_press_event_t* bp = (xcb_button_press_event_t*) event;
//        start_x_ = bp->root_x;
//        start_y_ = bp->root_y;
//        qDebug() << start_x_ << start_y_ << bp->event_x << bp->event_y;
//        // Record position.
//        break;
//      }
//      case XCB_BUTTON_RELEASE: {
//        qDebug() << "RELEASE";
//        this->uninstallFilter();
//        break;
//      }
//      case XCB_MOTION_NOTIFY: {
//        // Move target.
//        qDebug() << "MOVE";
//        xcb_motion_notify_event_t* motion = (xcb_motion_notify_event_t*) event;
//        const int x = motion->root_x;
//        const int y = motion->root_y;
//        qDebug() << x << y;
//        MoveWindow(target_->winId(), x, y);
//        break;
//      }
//      default: {
//        break;
//      }
//    }
  }
  Q_UNUSED(eventType);
  Q_UNUSED(message);
  Q_UNUSED(result);
  return false;
}

void MoveResizeEventFilter::startAction(QWidget* widget, Action action) {
  this->uninstallFilter();
  target_ = widget;
  action_ = action;
  QAbstractEventDispatcher::instance()->installNativeEventFilter(this);
}

void MoveResizeEventFilter::uninstallFilter() {
  qDebug() << Q_FUNC_INFO;
  if (target_ != nullptr) {
    QAbstractEventDispatcher::instance()->removeNativeEventFilter(this);
    target_ = nullptr;
  }
}
