/*
 * Copyright (C) 2017 ~ 2017 Deepin Technology Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CEF_TESTS_VLC_PLAYER_WINDOW_H
#define CEF_TESTS_VLC_PLAYER_WINDOW_H

#include <QFrame>
#include <QLabel>
#include <QPushButton>
#include <QShortcut>
#include <QSlider>

//#define ENABLE_QT_MEDIA_PLAYER

#ifdef ENABLE_QT_MEDIA_PLAYER
#include <QMediaPlayer>
typedef QMediaPlayer PlayerBackend;
#else
#include "service/backend/vlc_media_player.h"
typedef netease::VlcMediaPlayer PlayerBackend;
#endif

namespace netease {

class VlcPlayerWindow : public QFrame {
  Q_OBJECT
 public:
  explicit VlcPlayerWindow(QWidget* parent = nullptr);

 private:
  void initConnections();
  void initUI();
  void loadMedia(const QString& filepath);

  QPushButton* play_button_ = nullptr;
  QPushButton* pause_button_ = nullptr;
  QLabel* position_label_ = nullptr;
  QLabel* duration_label_ = nullptr;
  QSlider* slider_ = nullptr;
  PlayerBackend* player_ = nullptr;
  QShortcut* dialog_shortcut_ = nullptr;

  QStringList audio_files_;

 private slots:
  void showFileChooserDialog();

  void onSliderMoved(int value);

  void onPlayerDurationChanged(qint64 duration);
  void onPlayerPositionChanged(qint64 position);
  void onPlayerMediaStatusChanged(PlayerBackend::MediaStatus status);
};

}  // namespace netease

#endif  // CEF_TESTS_VLC_PLAYER_WINDOW_H
