// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#include "ne_enigma.h"

#include <string.h>
#include <string>
#include <QDebug>

#ifdef __cplusplus
extern "C" {
#endif
#include "include/serial_api.h"
#ifdef __cplusplus
}
#endif


namespace ne_enigma {

namespace {

// Ok status when calling orpheus encryption api for the first rand.
const int kStatusFirstRandOk = 2;

// Ok status when calling orpheus encryption api for the second rand.
const int kStatudSecondRandOk = 1;

}  // namespace

// Encrypt anonymous POST id.
QString EncodeAnonymousId(const QString& id) {
  unsigned int out_length = 0;
  const std::string input_data = id.toStdString();
  std::string result;
  int ret = 0;

  ret = OrpheusSerialAnonymousId(input_data.c_str(), input_data.length(),
                                 0, &out_length);
  if (ret == kStatusFirstRandOk) {
    result.resize(out_length);
    ret = OrpheusSerialAnonymousId(input_data.c_str(), input_data.length(),
                                   &result[0], &out_length);
    result.resize(out_length);
    if (ret == kStatudSecondRandOk) {
      return QString::fromStdString(result);
    }
  }
  qCritical() << "[ne_enigma]::EncodeAnonymousId() error!"
              << ", id: " << id;
  return "";
}

// Encrypt dfs id.
QString EncryptId(const QString& doc_id) {
  unsigned int out_length = 0;
  const std::string input_data = doc_id.toStdString();
  std::string result;
  int ret = 0;

  ret = OrpheusSerialDoc(input_data.c_str(), input_data.length(),
                         0, &out_length);
  if (ret == kStatusFirstRandOk) {
    result.resize(out_length);
    ret = OrpheusSerialDoc(input_data.c_str(), input_data.length(),
                           &result[0], &out_length);
    result.resize(out_length);
    if (ret == kStatudSecondRandOk) {
      return QString::fromStdString(result);
    }
  }
  qCritical() << "[ne_enigma]::EncryptId() error!"
              << ", doc_id: " << doc_id;
  return "";
}

// Encrypt POST data.
QString SerialData(const QString& data) {
  unsigned int out_length = 0;
  const std::string input_data = data.toStdString();
  std::string result;
  int ret = 0;

  ret = OrpheusSerialData(input_data.c_str(), input_data.length(),
                          0, &out_length);
  if (ret == kStatusFirstRandOk) {
    result.resize(out_length);
    ret = OrpheusSerialData(input_data.c_str(), input_data.length(),
                            &result[0], &out_length);
    result.resize(out_length);
    if (ret == kStatudSecondRandOk) {
      return QString::fromStdString(result);
    }
  }

  qCritical() << "[ne_enigma]::SerialData() error!"
              << ", data: " << data;
  return "";
}

}  // namespace ne_enigma
