// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#include "ui/models/playlist_model.h"

#include <QBrush>
#include <QDebug>
#include <QPixmap>
#include <QModelIndex>

#include "ui/models/song_info.h"

namespace netease {

PlaylistModel::PlaylistModel(QObject* parent)
  : QAbstractListModel(parent),
    playlist_(),
    current_play_row_(0),
    is_playing_(false) {
}

PlaylistModel::~PlaylistModel() {

}

void PlaylistModel::addSongs(const QList<SongInfo>& song_list) {
  this->beginResetModel();
  playlist_.append(song_list);
  this->endResetModel();
}

void PlaylistModel::clearPlaylist() {
  this->beginResetModel();
  playlist_.clear();
  // Also reset current playing index.
  this->endResetModel();
}

int PlaylistModel::rowOf(const QString& play_id) const {
  for (int i = 0; i < playlist_.length(); i ++) {
    if (playlist_.at(i).play_id == play_id) {
      return i;
    }
  }

  return -1;
}

int PlaylistModel::currentPlayRow() const {
    return current_play_row_;
}

QVariant PlaylistModel::data(const QModelIndex& index, int role) const {
  if (index.isValid() && (role == Qt::DisplayRole || role == Qt::EditRole)) {
    SongInfo info = playlist_.at(index.row());
    if (index.row() == current_play_row_) {
      if (is_playing_) {
        info.play_state = Playing;
      } else {
        info.play_state = Paused;
      }
    } else {
      info.play_state = Stoped;
    }
    QJsonObject obj;
    obj << info;

    return QVariant(obj);
  } else {
    return QVariant();
  }
}

void PlaylistModel::deleteSongs(const QStringList& play_id_list) {
  this->beginResetModel();

  for (int i = playlist_.length() - 1; i >= 0; --i) {
    if (play_id_list.contains(playlist_.at(i).play_id)) {
      playlist_.removeAt(i);
    }
  }

  this->endResetModel();
}

bool PlaylistModel::isEmpty() const {
  return playlist_.isEmpty();
}

void PlaylistModel::setCurrentRow(int row) {
  current_play_row_ = row;
  const QModelIndex top_left(index(current_play_row_));
  emit dataChanged(top_left, top_left);
}

SongInfo PlaylistModel::getSongInfo(int row) {
  SongInfo info;
  if (row >= 0 && row < playlist_.size()) {
    info = playlist_.at(row);
  }
  return info;
}

Qt::ItemFlags PlaylistModel::flags(const QModelIndex& index) const {
  Q_UNUSED(index)
  return Qt::ItemIsEditable | Qt::ItemIsEnabled | Qt::ItemIsSelectable;
}

int PlaylistModel::rowCount(const QModelIndex& parent) const {
  Q_UNUSED(parent);
  return playlist_.length();
}

void PlaylistModel::setPlayState(bool is_playing) {
  is_playing_ = is_playing;
  const QModelIndex top_left(index(current_play_row_));
  emit dataChanged(top_left, top_left);
}

}  // namespace netease
