// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#ifndef UI_DESKTOP_LYRICS_H_
#define UI_DESKTOP_LYRICS_H_

#include <QWidget>
class QCloseEvent;
class QFrame;
class QHideEvent;
class QLabel;
class QMouseEvent;
class QWheelEvent;

#include "service/media_player.h"

namespace netease {

class DesktopLyricsUnlockWindow;
class IconButton;
class LrcCompoundFrame;

class DesktopLyrics : public QWidget {
  Q_OBJECT

 public:
  DesktopLyrics(QWidget* parent = nullptr);
  ~DesktopLyrics();

  // Check desktop lyrics window is locked or not.
  bool locked() const;

  void setTopMost(bool topmost);

  // Lock/unlock desktop lyrics.
  void setLocked(bool locked);

  void setShowWindow(bool show);

  // Update tooltips of playback control buttons.
  void updateTooltips(const QString& prev_tooltip,
                      const QString& play_pause_tooltip,
                      const QString& next_tooltip);

 public slots:
  // Restore window from last position.
  void show();

 protected:
  // Prevents window from being closed by pressing Alt+F4.
  void closeEvent(QCloseEvent* event) override;

  bool eventFilter(QObject* obj, QEvent* event) override;

  void hideEvent(QHideEvent* event) override;
  void mousePressEvent(QMouseEvent* event) override;
  void mouseReleaseEvent(QMouseEvent* event) override;

  // Notify cursor motion recorder service to update |rect|.
  void moveEvent(QMoveEvent* event) override;

  // Emit onWindowResized() signal.
  void resizeEvent(QResizeEvent* event) override;

  void showEvent(QShowEvent* event) override;

  // Override wheelEvent to change lrc time offset.
  void wheelEvent(QWheelEvent* event) override;

 private:
  void initConnections();
  void initUI();

  bool isMouseHoverChild();

  QPoint lastWindowPos() const;
  void setControlButtonsVisible(bool visible);
  void setUnlockWindowVisible(bool visible);

  // To mark desktop lyrics window is locked or not.
  bool is_locked_;

  bool is_topmost_;

  // Desktop lyrics window is visible or not.
  bool is_lyrics_visible_;

  QLabel* background_label_;

  IconButton* detail_button_;
  QLabel* sep1_;
  IconButton* prev_button_;
  IconButton* play_pause_button_;
  IconButton* next_button_;
  QLabel* sep2_;
  IconButton* settings_button_;
  IconButton* lock_button_;
  QLabel* sep3_;
  IconButton* minus_offset_button_;
  IconButton* plus_offset_button_;
  QLabel* sep4_;
  IconButton* close_button_;
  QFrame* control_buttons_container_;

  LrcCompoundFrame* lrc_compound_frame_ = nullptr;
  DesktopLyricsUnlockWindow* unlock_window_ = nullptr;
  QTimer* hover_poll_timer_ = nullptr;

  // To record cursor enter/leave timestamp.
  int cursor_enter_time_;
  int cursor_leave_time_;

 private slots:
  void handleCursorEnterEvent();
  void handleCursorEnterEventDelay();
  void handleCursorLeaveEvent();
  void handleCursorLeaveEventDelay();

  // Check whether global cursor is in current window.
  void handleHoverPollTimerTimeout();

  // Resize current window based when font size of lrc changed.
  void handleLrcFontSizeChanged();

  // Deactive prev_button_ and next_button_ is current lrc is empty.
  void handleLrcUpdated();

  // Update play_pause_button_ icons based on status of media player.
  void handleMediaPlayerStateChanged(MediaPlayer::State state);

  void handlePlayPauseButtonClicked();

  void updateSkin();

  void moveUnlockWindowAlong();

  void applyInputTransparent();
};

}  // namespace netease

#endif  // UI_DESKTOP_LYRICS_H_
