// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#include "ui/views/desktop_lyrics_unlock_window.h"

#include <QHBoxLayout>
#include <QLabel>
#include <QPushButton>
#include <QVBoxLayout>

#include "application.h"
#include "service/skin_manager.h"
#include "ui/utils/xutil.h"
#include "ui/views/widgets/flat_button.h"

namespace netease {

namespace {

const char kStyleSheet[] = ":/styles/desktop-lyrics-unlock-window.css";

}  // namespace

DesktopLyricsUnlockWindow::DesktopLyricsUnlockWindow(QWidget* parent)
    : QFrame(parent) {

  initUI();
  updateSkin();
  initConnections();
}

void DesktopLyricsUnlockWindow::initConnections() {
  connect(unlock_button_, &QPushButton::clicked,
          this, &DesktopLyricsUnlockWindow::unlocked);
  connect(app->skin_manager, &SkinManager::onUpdated,
          this, &DesktopLyricsUnlockWindow::updateSkin);
}

void DesktopLyricsUnlockWindow::initUI() {
  QLabel* background_label = new QLabel(this);
  background_label->setObjectName("background-label");

  unlock_button_ = new FlatButton();
  unlock_button_->setObjectName("unlock-button");
  unlock_button_->setFixedSize(21, 20);

  QHBoxLayout* hbox = new QHBoxLayout();
  hbox->setContentsMargins(0, 0, 0, 0);
  hbox->setSpacing(0);
  hbox->addWidget(unlock_button_, Qt::AlignCenter);

  QVBoxLayout* layout = new QVBoxLayout();
  layout->setContentsMargins(0, 0, 0, 0);
  layout->setSpacing(0);
  layout->addLayout(hbox);

  this->setLayout(layout);
  this->setContentsMargins(0, 0, 0, 0);
  this->setFixedSize(40, 36);
  this->setObjectName("unlock-window");
  setAttribute(Qt::WA_TranslucentBackground, true);
  setWindowFlags(Qt::FramelessWindowHint);
}

void DesktopLyricsUnlockWindow::updateSkin() {
  setStyleSheet(app->skin_manager->formatStyleFile(kStyleSheet));
}

void DesktopLyricsUnlockWindow::showEvent(QShowEvent* event) {
  utils::SetWindowTypeHint(this, utils::X11WindowType::Dock);
  utils::SkipTaskbarPager(this);
  QFrame::showEvent(event);
}

}  // namespace netease
