// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#include "ui/views/web_view.h"

#include <QCloseEvent>
#include <QDebug>
#include <QDesktopWidget>
#include <QMouseEvent>
#include <QPoint>
#include <QResizeEvent>

#include "application.h"
#include "base/consts.h"
#include "base/file_util.h"
#include "ui/utils/xutil.h"

namespace netease {

namespace {

//const char kWebFrameRateFile[] = ":/misc/display-webview-frame-rate.js";
//const char kMvControllerFile[] = ":/misc/fullscreen-video-controller.js";

const int kBorderWidth = 10;

}  // namespace

WebView::WebView()
    : is_main_window(false),
      left_pressed_(false),
      resizable_(false),
      resizable_disabled_(false),
      last_window_state_(Qt::WindowNoState) {
  // Remove titlebar and border.
  this->setWindowFlags(Qt::Window | Qt::FramelessWindowHint);

  // Setup transparent background, thus displays round corner in webview.
//  this->setAttribute(Qt::WA_TranslucentBackground);
  this->setContentsMargins(0, 0, 0, 0);

  this->initConnections();
}

WebView::~WebView() {
}

void WebView::closeEvent(QCloseEvent* event) {
  Q_UNUSED(event);
  qDebug() << "[WebView]::closeEvent()";
  emit onClose();
//  QFrame::closeEvent(event);
//  deleteLater();
}

void WebView::closeWindow() {
  this->close();
}

void WebView::disableResizable() {
  resizable_disabled_ = true;
}

void WebView::enterFullscreen() {
  if (!this->isVisible()) {
    return;
  }
  last_window_state_ = this->windowState();
  this->showFullScreen(true);
}

bool WebView::eventFilter(QObject* obj, QEvent* event) {
  return QFrame::eventFilter(obj, event);
}
void WebView::exitFullscreen() {
  if (!this->isVisible()) {
    return;
  }

  utils::ShowFullscreenWindow(this, false);
  switch (last_window_state_) {
  case Qt::WindowMaximized:
    this->showMaximized();
    break;

  case Qt::WindowNoState:
    this->showNormal();
    break;

  default:
    // Ignore this state.
    break;
  }
}

void WebView::handleCefFullscreenChanged(bool fullscreen) {
  qDebug() << "[WebView]::handleCefFullscreenChanged():" << fullscreen;
  if (fullscreen) {
    enterFullscreen();
  } else {
    exitFullscreen();
  }
}

void WebView::initConnections() {
}

void WebView::load(const QUrl& url) {
  Q_UNUSED(url);
}

void WebView::moveEvent(QMoveEvent* event) {
  QFrame::moveEvent(event);
}
void WebView::moveToCenter() {
  move(app->desktop()->screen()->rect().center() - rect().center());
}

void WebView::moveWindow() {
  utils::MoveWindow(this);
}

void WebView::raiseWindow() {
  this->show();
  this->activateWindow();
  this->raise();
}

void WebView::resizeEvent(QResizeEvent* event) {
  emit this->windowStateSizeChanged();
  QFrame::resizeEvent(event);
}

void WebView::setResizable(bool resizable) {
  if (resizable_disabled_) {
    qWarning() << "[WebView]::setResizable() current window has fixed size!";
    return;
  }

  resizable_ = resizable;
  if (! resizable) {
    qDebug() << "[WebView]::setResizable() will emit onSizeWindowDone() signal";
    emit onSizeWindowDone();
  }
}

void WebView::resizeWindow(const QString& corner) {
  utils::GrabFocus(this);
  this->setResizable(true);
  utils::ResizeWindow(this, corner);
}

void WebView::setTopMost(bool topmost) {
  Qt::WindowFlags flags = this->windowFlags();
  if (topmost) {
    flags = flags | Qt::WindowStaysOnTopHint;
  } else if (flags & Qt::WindowStaysOnTopHint){
    flags = flags ^ Qt::WindowStaysOnTopHint;
  }
  const bool visible = this->isVisible();
  this->setWindowFlags(flags);
  if (visible) {
    this->show();
  }
}

void WebView::showFullScreen(bool fullscreen) {
  utils::ShowFullscreenWindow(this, fullscreen);
  this->show();
  this->activateWindow();
  this->raise();
}

void WebView::showMaximized() {
  utils::ShowMaximizedWindow(this);
  this->show();
  this->activateWindow();
  this->raise();
}

void WebView::showMinimized() {
  utils::ShowMinimizedWindow(this, true);
}

void WebView::showNormal() {
//  utils::ShowMinimizedWindow(this, false);
//  utils::ShowFullscreenWindow(this, false);
  utils::ShowNormalWindow(this);
  this->show();
  this->activateWindow();
  this->raise();
}

}  // namespace netease
