// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#include "ui/views/mini_player_info_panel.h"

#include <QDebug>
#include <QHBoxLayout>
#include <QLabel>

#include "application.h"
#include "service/skin_manager.h"
#include "ui/views/mini_player_consts.h"

namespace netease {

namespace {

const char kStyleSheetFile[] = ":/styles/mini-player-info-panel.css";

}  // namespace

MiniPlayerInfoPanel::MiniPlayerInfoPanel(QWidget* parent) : QFrame(parent) {
  initUI();
  updateSkin();
  initConnections();
}

void MiniPlayerInfoPanel::initConnections() {
  connect(app->skin_manager, &SkinManager::onUpdated,
          this, &MiniPlayerInfoPanel::updateSkin);
}

void MiniPlayerInfoPanel::initUI() {
  title_label_ = new QLabel();
  title_label_->setObjectName("title-label");
  title_label_->setAlignment(Qt::AlignRight);

  artist_label_ = new QLabel();
  artist_label_->setObjectName("artist-label");
  artist_label_->setAlignment(Qt::AlignLeft);

  // These three part of background are laminated
  QFrame* bg_left_frame = new QFrame();
  bg_left_frame->setFixedHeight(kInfoPanelHeight);
  bg_left_frame->setObjectName("info-panel-left-bg");
  QHBoxLayout* bg_left_layout = new QHBoxLayout(this);
  bg_left_layout->setContentsMargins(0, 0, 0, 0);
  bg_left_layout->addWidget(bg_left_frame);

  QFrame* bg_right_frame = new QFrame();
  bg_right_frame->setFixedHeight(kInfoPanelHeight);
  bg_right_frame->setObjectName("info-panel-right-bg");
  QHBoxLayout* bg_right_layout = new QHBoxLayout(bg_left_frame);
  bg_right_layout->setContentsMargins(0, 0, 0, 0);
  bg_right_layout->addWidget(bg_right_frame);

  QFrame* bg_mid_frame = new QFrame();
  bg_mid_frame->setFixedHeight(kInfoPanelHeight);
  bg_mid_frame->setObjectName("info-panel-mid-bg");
  QHBoxLayout* bg_mid_layout = new QHBoxLayout(bg_right_frame);
  bg_mid_layout->setContentsMargins(0, 0, 0, 0);
  bg_mid_layout->addWidget(bg_mid_frame);

  QHBoxLayout* layout = new QHBoxLayout(bg_mid_frame);
  layout->setContentsMargins(0, 0, 0, 0);
  layout->setSpacing(0);
  layout->addStretch(1);
  layout->addWidget(title_label_);
  layout->addWidget(artist_label_);
  layout->addStretch(1);

  this->resize(kWindowMinimumWidth, kInfoPanelHeight);
}

void MiniPlayerInfoPanel::updateContent(const QString& title,
                                        const QString& artist) {
  title_label_->setText(title);
  artist_label_->setText(QString(" - ") + artist);
}

void MiniPlayerInfoPanel::updateSkin() {
  setStyleSheet(app->skin_manager->formatStyleFile(kStyleSheetFile));
}

}  // namespace netease
