// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#ifndef UI_VIEWS_MINI_PLAYER_PLAYLIST_H_
#define UI_VIEWS_MINI_PLAYER_PLAYLIST_H_

#include <QFrame>
class QModelIndex;
class QListView;

namespace netease {

class PlaylistDelegate;
class PlaylistModel;
struct SongInfo;

class MiniPlayerPlaylist : public QFrame {
  Q_OBJECT

 public:
  explicit MiniPlayerPlaylist(QWidget* parent = nullptr);

  void addSongs(const QList<SongInfo>& song_list);

  // Remove all songs in playlist.
  void clearPlaylist();

  // Get current song info. If not set, returns an empty SongInfo object.
  SongInfo currentSongInfo() const;

  // Remove songs from playlist.
  void deleteSongs(const QStringList& play_id_list);

  void setCurrentPlay(const QString& play_id);

  void setPlayState(bool is_playing);

 signals:
  // Switch to new song when that row is double clicked.
  void changeCurrentPlay(const QString& play_id);

 private slots:
  void handleContextMenu(QPoint pos);

  void handleListViewDoubleClicked(const QModelIndex& index);

  void updateSkin();

 private:
  void initConnection();
  void initUI();

 private:
  QListView* playlist_view_;
  PlaylistDelegate* playlist_delegate_;
  PlaylistModel* playlist_model_;
};

}  // namespace netease

#endif  // UI_VIEWS_MINI_PLAYER_PLAYLIST_H_
