// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#include "ui/views/mini_player_title.h"

#include <QDebug>
#include <QEvent>
#include <QLabel>
#include <QResizeEvent>
#include <QVBoxLayout>

#include "application.h"
#include "base/file_util.h"
#include "service/skin_manager.h"
#include "ui/views/mini_player_consts.h"

namespace netease {

namespace {

const char kStyleSheetFile[] = ":/styles/mini-player-title.css";

// Margin value at each horizontal side.
const int kHorizontalSideMargin = 15;

}  // namespace

MiniPlayerTitle::MiniPlayerTitle(QWidget* parent)
    : QFrame(parent),
      widget_size_(kWindowMinimumWidth, 0) {
  initUI();

  connect(app->skin_manager, &SkinManager::onUpdated,
          this, &MiniPlayerTitle::updateStyle);
}

void MiniPlayerTitle::enterEvent(QEvent* event) {
  Q_UNUSED(event);
  emit mouseEntered();
}

void MiniPlayerTitle::initUI() {
  title_label_ = new QLabel();
  title_label_->setObjectName("title-label");
  title_label_->setAlignment(Qt::AlignHCenter);

  artist_label_ = new QLabel();
  artist_label_->setObjectName("artist-label");
  artist_label_->setAlignment(Qt::AlignHCenter);

  QVBoxLayout* layout = new QVBoxLayout();
  layout->setContentsMargins(0, 0, 0, 0);
  layout->setSpacing(2);
  layout->addWidget(title_label_);
  layout->addWidget(artist_label_);

  setLayout(layout);

  setFixedHeight(kTopPanelHeight);
  this->setContentsMargins(kHorizontalSideMargin, 0, kHorizontalSideMargin, 0);
  this->updateStyle();
}

void MiniPlayerTitle::updateContent(const QString& title,
                                    const QString& artist) {
  // Including side margins.
  const int width = widget_size_.width() - 2 * kHorizontalSideMargin;

  const QFontMetrics title_font_metrics(title_label_->fontMetrics());
  title_label_->setText(
      title_font_metrics.elidedText(title, Qt::ElideRight, width));

  const QFontMetrics artist_font_metrics(artist_label_->fontMetrics());
  artist_label_->setText(
      artist_font_metrics.elidedText(artist, Qt::ElideRight, width));
}

void MiniPlayerTitle::updateStyle() {
  setStyleSheet(app->skin_manager->formatStyleFile(kStyleSheetFile));
}

void MiniPlayerTitle::resizeEvent(QResizeEvent* event) {
  this->widget_size_ = event->size();
  QFrame::resizeEvent(event);
}

}  // namespace netease
