// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#ifndef UI_WEB_VIEW_H_
#define UI_WEB_VIEW_H_

#include <QFrame>
class QCloseEvent;
class QCursor;
class QFocusEvent;
class QMouseEvent;
class QPoint;
class QResizeEvent;
class QCefWebView;

namespace netease {

class WebViewEventDelegate;

// WebView is a contains to hold QCefView widget.
class WebView : public QFrame {
  Q_OBJECT

 public:
  WebView();
  ~WebView();

  bool is_main_window;

  // Set current windows has fixed size.
  void disableResizable();

  // Set window resizable.
  void setResizable(bool resizable);

 signals:
  // Emitted when closed.
  void onClose();

  // Emitted when webview window grabs focus.
  void onFocusIn();

  // Emitted when webview window loses focus.
  void onFocusOut();

  // Emitted after resizing window operation is done.
  void onSizeWindowDone();

  void windowStateSizeChanged();

  // Move window to new position. New position is calculated based on cursor
  // position.
  void moveWindow();

  // Resize window at |position| corner.
  void resizeWindow(const QString& corner);

 public slots:
  void enterFullscreen();
  void exitFullscreen();

  // Implement for QWebView.
  void load(const QUrl& url);

  // Set this window to be a top-level activated window first. Or else it is
  // not raised on top in some window managers (like deepin-mutter).
  void raiseWindow();

  void setTopMost(bool topmost);

  void setZoomLevel(double level);

  // Override these slots in QWidget to fix window state chaos in Qt.
  void showFullScreen(bool fullscreen);
  void showMaximized();
  void showMinimized();
  void showNormal();

 protected:
  // To emit onClose() signal.
  void closeEvent(QCloseEvent* event) override;

  bool eventFilter(QObject* obj, QEvent* event) override;

  // conditions to emit windowStateSizeChanged
  void resizeEvent(QResizeEvent*) override;

 private:
  void initConnections();

  // Update zoom level if needed.
  void updateWebViewZoomLevel();

  QCefWebView* qcef_webview_ = nullptr;

  // Currently only support left press to drag window.
  bool left_pressed_;

  // To mark whether current window is resizable by mouse.
  bool resizable_;

  // To set whether current window has fixed size.
  bool resizable_disabled_;

  Qt::WindowStates last_window_state_;

  WebViewEventDelegate* event_delegate_ = nullptr;

 private slots:
  void handleCefFullscreenChanged(bool fullscreen);

  void handleMoveWindow();
  void handleResizeWindow(const QString& corner);
};

}  // namespace netease

#endif  // UI_WEB_VIEW_H_
