// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#include "ui/views/widgets/colored_icon_button.h"

#include <QDebug>
#include <QFile>
#include <QImage>

#include "application.h"
#include "service/skin_manager.h"
#include "ui/views/mini_player_consts.h"

namespace netease {

namespace {

const char kColorButtonsFile[] = "SKIN_ROOT/btn/color_btns.png";

const QImage& ColorizeImage(const QString& filepath, const QColor& color) {
  // Cache generated image.
  static QColor current_color;
  static QImage current_image;
  if (color == current_color) {
    return current_image;
  }

  current_color = color;
  current_image.load(filepath);
  const int red = color.red();
  const int green = color.green();
  const int blue = color.blue();
  for (int y = 0; y < current_image.height(); ++y) {
    QRgb* destrow = (QRgb*) current_image.scanLine(y);
    for (int x = 0; x < current_image.width(); ++x) {
      if (qAlpha(destrow[x]) > 0) {
        destrow[x] = qRgba(red, green, blue, qAlpha(destrow[x]));
      }
    }
  }
  return current_image;
}

}  // namespace

ColoredIconButton::ColoredIconButton(int offset_x,
                                     int offset_y,
                                     int hover_offset_x,
                                     int hover_offset_y,
                                     int width,
                                     int height,
                                     QWidget* parent)
    : IconButton(kCollectionButtons,
                 offset_x,
                 offset_y,
                 kCollectionButtons,
                 hover_offset_x,
                 hover_offset_y,
                 width,
                 height,
                 parent) {
  this->updateButtonStyle();

  connect(app->skin_manager, &SkinManager::onUpdated,
          this, &ColoredIconButton::updateButtonStyle);
}

void ColoredIconButton::updateButtonStyle() {
  const QString color_buttons(
      app->skin_manager->absoluteSkinPath(kColorButtonsFile));
  if (QFile::exists(color_buttons)) {
    // Use colored button.
    const QImage& image(
        ColorizeImage(color_buttons, app->skin_manager->buttonColor()));
    QPixmap pixmap(QPixmap::fromImage(image));
    normal_icon_ = QIcon(pixmap.copy(offset_x_, offset_y_,
                                     width_, height_));
    hover_icon_ = QIcon(pixmap.copy(hover_offset_x_, hover_offset_y_,
                                    width_, height_));
    this->setIcon(normal_icon_);
  } else {
    // Use default button.
    IconButton::updateButtonStyle();
  }
}

void ColoredIconButton::updateIconPaths(int offset_x,
                                        int offset_y,
                                        int hover_offset_x,
                                        int hover_offset_y) {
  IconButton::updateIconPaths(icon_path_, offset_x, offset_y,
                              hover_icon_path_, hover_offset_x, hover_offset_y);
  this->updateButtonStyle();
}

}  // namespace netease