// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#include "ui/views/widgets/icon_button.h"

#include <QEvent>
#include <QPixmap>

#include "application.h"
#include "service/skin_manager.h"

namespace netease {

IconButton::IconButton(const QString& icon_path,
                       int offset_x,
                       int offset_y,
                       const QString& hover_icon_path,
                       int hover_offset_x,
                       int hover_offset_y,
                       int width,
                       int height,
                       QWidget* parent)
    : FlatButton(parent),
      icon_path_(icon_path),
      offset_x_(offset_x),
      offset_y_(offset_y),
      hover_icon_path_(hover_icon_path),
      hover_offset_x_(hover_offset_x),
      hover_offset_y_(hover_offset_y),
      width_(width),
      height_(height) {

  this->setFlat(true);
  this->updateButtonStyle();
  QSize size(width_, height_);
  this->setIconSize(size);
  this->setFixedSize(size);

  QObject::connect(app->skin_manager, &SkinManager::onUpdated,
                   this, &IconButton::updateButtonStyle);
}

QPixmap IconButton::normalPixmap() const {
  return normal_icon_.pixmap(width_, height_);
}

QPixmap IconButton::hoverPixmap() const {
  return hover_icon_.pixmap(width_, height_);
}

void IconButton::enterEvent(QEvent* event) {
  setIcon(hover_icon_);
  FlatButton::enterEvent(event);
}

void IconButton::leaveEvent(QEvent* event) {
  setIcon(normal_icon_);
  FlatButton::leaveEvent(event);
}

void IconButton::updateIconPaths(const QString& icon_path,
                                 int offset_x,
                                 int offset_y,
                                 const QString& hover_icon_path,
                                 int hover_offset_x,
                                 int hover_offset_y) {
  icon_path_ = icon_path;
  hover_icon_path_ = hover_icon_path;
  offset_x_ = offset_x;
  offset_y_ = offset_y;
  hover_offset_x_ = hover_offset_x;
  hover_offset_y_ = hover_offset_y;
  updateButtonStyle();
}

void IconButton::updateButtonStyle() {
  const QString icon_path(app->skin_manager->absoluteSkinPath(icon_path_));
  normal_icon_ = QIcon(QPixmap(icon_path).copy(offset_x_,
                                               offset_y_,
                                               width_,
                                               height_));
  const QString hover_icon_path(
      app->skin_manager->absoluteSkinPath(hover_icon_path_));
  hover_icon_ = QIcon(QPixmap(hover_icon_path).copy(hover_offset_x_,
                                                    hover_offset_y_,
                                                    width_,
                                                    height_));
  this->setIcon(normal_icon_);
}

}  // namespace netease
