// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#ifndef UI_VIEWS_WIDGETS_ICON_BUTTON_H_
#define UI_VIEWS_WIDGETS_ICON_BUTTON_H_

#include "ui/views/widgets/flat_button.h"

class QEvent;
class QIcon;

namespace netease {

// A FlatButton which supports backgound icon clipping.
class IconButton : public FlatButton {
  Q_OBJECT

 public:
  IconButton(const QString& icon_path,
             int offset_x,
             int offset_y,
             const QString& hover_icon_path,
             int hover_offset_x,
             int hover_offset_y,
             int width,
             int height,
             QWidget* parent = nullptr);
  QPixmap normalPixmap() const;
  QPixmap hoverPixmap() const;

 public slots:
  // Update button icons.
  void updateIconPaths(const QString& icon_path,
                       int offset_x,
                       int offset_y,
                       const QString& hover_icon_path,
                       int hover_offset_x,
                       int hover_offset_y);

 protected:
  // Hoverride these two event handlers to implements hover effect.
  void enterEvent(QEvent* event) override;
  void leaveEvent(QEvent* event) override;

  QIcon normal_icon_;
  QIcon hover_icon_;

  QString icon_path_;
  int offset_x_;
  int offset_y_;
  QString hover_icon_path_;
  int hover_offset_x_;
  int hover_offset_y_;

  // Width and height of clipped image.
  int width_;
  int height_;

 protected slots:
  // Update button icons based on current skin.
  void updateButtonStyle();
};

}  // namespace netease

#endif  // UI_VIEWS_WIDGETS_ICON_BUTTON_H_
