// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#include "ui/views/widgets/popup_menu.h"

#include <QDebug>
#include <QMouseEvent>
#include <QWidgetAction>

#include "application.h"
#include "service/skin_manager.h"
#include "ui/views/widgets/popup_menu_item.h"

namespace netease {

namespace {

const char kStyleSheetFile[] = ":/styles/popup-menu.css";

}  // namespace

PopupMenu::PopupMenu(QWidget* parent) : QMenu(parent) {
  initConnections();
  updateSkin();
}

QAction* PopupMenu::createAction(const QPixmap& icon, const QString& text,
                                 const QPixmap& check_icon) {
  QWidgetAction* action = new QWidgetAction(this);

  PopupMenuItem* menu_item = new PopupMenuItem(icon, text, check_icon);
  action->setDefaultWidget(menu_item);
  return action;
}

void PopupMenu::initConnections() {
  connect(app->skin_manager, &SkinManager::onUpdated,
          this, &PopupMenu::updateSkin);
}

void PopupMenu::updateSkin() {
  this->setStyleSheet(app->skin_manager->formatStyleFile(kStyleSheetFile));
}

}  // namespace netease