// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#ifndef UI_VIEWS_WIDGETS_POPUP_MENU_STYLE_H
#define UI_VIEWS_WIDGETS_POPUP_MENU_STYLE_H

#include <QMenu>
#include <QProxyStyle>

namespace netease {

const char kCheckedIconCacheKey[] = "menu_checked_icon";

class PopupMenuStyle : public QProxyStyle {
  Q_OBJECT

 public:
  PopupMenuStyle();

  int styleHint(StyleHint hint,
                const QStyleOption* option = 0,
                const QWidget* widget = 0,
                QStyleHintReturn* returnData = 0) const;
  int pixelMetric(PixelMetric metric,
                  const QStyleOption* option = 0,
                  const QWidget* widget = 0) const;
  void drawControl(ControlElement control,
                   const QStyleOption* option,
                   QPainter* painter,
                   const QWidget* widget) const;
  QSize sizeFromContents(ContentsType type,
                         const QStyleOption* option,
                         const QSize& contentsSize,
                         const QWidget* widget = 0) const;
 private:
  void drawScroller(const QStyleOption* option,
                    QPainter* painter) const;
  void drawMenuItem(const QStyleOption* option,
                    QPainter* painter) const;
  void drawMenuItemIcon(const QStyleOptionMenuItem* menu_item,
                        QPainter* painter) const;
  void drawMenuItemText(const QStyleOptionMenuItem* menu_item,
                        QPainter* painter) const;
  void drawMenuItemSubIcon(const QStyleOptionMenuItem* menu_item,
                           QPainter* painter) const;
  void drawMenuItemCheckedIcon(const QStyleOptionMenuItem* menu_item,
                               QPainter* painter) const;

  QSize getItemSize(const QStyleOption* option, const QSize& size) const;

  QPixmap getArrowPixmap(const QStyle::State state) const;
  QPixmap getItemBGPixmap(const QStyle::State state) const;
  QPixmap getIconPixmap(const QStyle::State state,
                        const QIcon& icon,
                        const QSize& size) const;
  QString absoluteSkinPath(const QString& path) const;
};

}  // namespace netease

#endif // UI_VIEWS_WIDGETS_POPUP_MENU_STYLE_H
