// Copyright (c) 2015 Deepin Ltd. All rights reserved.
// Use of this source is governed by Commercial License that can be found
// in the LICENSE file.

#ifndef UI_VIEWS_WIDGETS_VOLUME_WINDOW_H_
#define UI_VIEWS_WIDGETS_VOLUME_WINDOW_H_

#include <QFrame>
class QLabel;
class QMouseEvent;

namespace netease {

class VolumeWindow : public QFrame {
  Q_OBJECT

 public:
  explicit VolumeWindow(QWidget* parent = nullptr);

  // Show volume window at |pos|.
  void show(const QPoint& pos);

  int volume() const { return volume_; }

 signals:
  // Emitted when user moves volume slide.
  void volumeChanged(int volume);

 public slots:
  void setVolume(int volume);

 protected:
  // Hide volume window when current window loses focus.
  bool event(QEvent* event) override;

  // Override these event handlers to change volume when mouse is pressed and
  // moved.
  void mouseMoveEvent(QMouseEvent* event) override;
  void mousePressEvent(QMouseEvent* event) override;
  void mouseReleaseEvent(QMouseEvent* event) override;

 private:
  void initUI();
  void initConnections();

  // Update handle_button_ position based on current volume.
  void updateSlidePos();

  QLabel* background_label_;
  QLabel* rail_label_;
  QLabel* slide_label_;

  int volume_;

  // Show reversed background image.
  bool is_reversed_;

  bool volume_changable_;

 private slots:
  void updateSkin();
};

}  // namespace netease

#endif  // UI_VIEWS_WIDGETS_VOLUME_WINDOW_H_
