#!/bin/bash

readonly work_dir=`pwd`
readonly output_root=$work_dir/installs
readonly qcef_out=$work_dir/qcef_out
readonly install_root=/opt/netease/netease-cloud-music
readonly tools_dir=$(cd `dirname $0`; pwd)
readonly NCM_root=$tools_dir/../

InstallDepends() {
    sudo apt install \
                debhelper \
                bison \
                cmake \
                flex \
                libbison-dev \
                libfl-dev \
                libfontconfig1-dev \
                libglib2.0-dev \
                libharfbuzz-dev \
                libnspr4-dev \
                libnss3-dev \
                libqt5webchannel5-dev \
                libqt5x11extras5-dev \
                libsqlite3-dev \
                libtag1-dev \
                libvlc-dev \
                libx11-dev \
                libxext-dev \
                libxss-dev \
                libxtst-dev \
                pkg-config \
                zlib1g-dev \
                libgconf2-dev \
                libpango1.0-dev \
                devscripts \
                golang-go \
                git  || return 1
}

BuildQCef() {
    if [ -d "$qcef_out" ];then
        return 0
    fi

    if [ ! -d "./qcef" ];then
        git clone --depth 1 https://github.com/linuxdeepin/qcef.git
        cd qcef
        git submodule update --init --recursive
	    git submodule update --remote
    else
        cd qcef
    fi

    sudo mk-build-deps -i debian/control

    if [ ! -d "./build" ];then
        mkdir build
    fi
    cd build
    cmake ../ -DCMAKE_BUILD_TYPE=Release -DCMAKE_INSTALL_PREFIX=/usr -DQCEF_INSTALL_PATH=$install_root/libs
    make $@ || return 1
    make install DESTDIR=$qcef_out  || return 1
    mv $qcef_out$install_root/libs $qcef_out/lib
    rm $qcef_out/lib/pkgconfig/libqcef.pc
    cat > $qcef_out/lib/pkgconfig/libqcef.pc << EOF
prefix=$qcef_out
exec_prefix=$qcef_out
libdir=$qcef_out/lib
includedir=$qcef_out/usr/include

Name: QCEF
Description: Qt5 binding of CEF3
Version: 1.1.4
Libs: -L$qcef_out/lib -lqcef
Cflags: -I$qcef_out/usr/include/qcef
EOF
}

BuildNCM() {
    if [ ! -d "./build_NCM" ];then
        mkdir build_NCM
    fi
    cd build_NCM
    export PKG_CONFIG_PATH=$qcef_out/lib/pkgconfig
    cmake $NCM_root -DCMAKE_INSTALL_PREFIX=/usr -DCMAKE_BUILD_TYPE=Release
    make $@ || return 1
    make install DESTDIR=$output_root || return 1
}

CopyDependFiles() {
    lib_path=$output_root$install_root/libs
    system_lib_path=/usr/lib/`dpkg-architecture -qDEB_HOST_MULTIARCH`
    ncm_file=$output_root/usr/bin/netease-cloud-music
    if [ ! -d "$lib_path" ];then
        mkdir -p $lib_path
    fi

# 不依赖工具处理程序依赖（未完成，不可使用）
#     cp -H `ldd $qcef_out/lib/libqcef.so|grep Qt5|cut -d' ' -f 3|xargs` $lib_path || return 1
#     cp -H `ldd $ncm_file|grep Qt5|cut -d' ' -f 3|xargs` $lib_path || return 1

#     # 复制Qt插件
#     qt_plugins=`qmake -query QT_INSTALL_PLUGINS`
#     qt_plugin_out=$lib_path/plugins

#     echo "find Qt plugins directory: $qt_plugins"

#     if [ ! -d "$qt_plugin_out" ];then
#         mkdir -p $qt_plugin_out
#     fi

#     plugin_dir_list=( \
#         "audio" "iconengines" "imageformats" \
#         "mediaservice" "platforminputcontexts" \
#         "sqldrivers" "platformthemes" "sensorgestures" \
#         "sensors" "styles" "xcbglintegrations" "platforms")

#     for i in "${plugin_dir_list[@]}"; do
#         # 复制插件目录
#         echo "copy directory \"$qt_plugins/$i\" to \"$qt_plugin_out\""
#         cp -r $qt_plugins/$i $qt_plugin_out
#         # 复制插件依赖的Qt库
#         for j in $qt_plugin_out/$i/*; do
#             cp -H `ldd $j|grep Qt5|cut -d' ' -f 3|xargs` $lib_path
#         done
#     done

#     # 移除Qt库尾部的版本号，让其以.so结尾
#     for i in $lib_path/*; do
#         if [[ "$i" == *so.5 ]];then
#             mv $i ${i:0:-2}
#         fi
#     done

#     # 生成Qt库配置文件，正确指定Qt插件位置
#     cat > $output_root$install_root/qt.conf << EOF
# [Paths]
# Libraries="lib"
# Plugins="lib/plugins"
# EOF

#     # 处理可执行程序
#     # 将二进制文件移动到安装目录
#     mv $ncm_file $output_root$install_root
#     # 生成可执行脚本
#     cat > $ncm_file << EOF
# #!bin/sh

# export LD_LIBRARY_PATH=$install_root/lib:$install_root/lib/qcef
# $install_root/netease-cloud-music \$@
# EOF
#     # 添加可执行权限
#     chmod +x $ncm_file

    # 使用开源工具复制二进制依赖
    export GOPATH=$work_dir/go
    go get -v github.com/Iceyer/bale

    $GOPATH/bin/bale -out $output_root$install_root -path $ncm_file -qt \
    -plugin $system_lib_path:$system_lib_path/nss \
    -plugin $system_lib_path:$system_lib_path/vlc \
    -plugin $qcef_out/lib:$qcef_out/lib/libqcef.so \
    -plugin $qcef_out/lib:$qcef_out/lib/qcef/libcef.so \
    -plugin $system_lib_path:$system_lib_path/libpangoft2-1.0.so.0 \

    rm -f $lib_path/libqcef.so* $lib_path/libcef.so*
    cp -d $qcef_out/lib/libqcef.so* $lib_path || return 1
    cp -r $qcef_out/lib/qcef $lib_path || return 1

    # 删除二进制，添加一个符号链接
    rm $ncm_file
    ln -s $install_root/netease-cloud-music.bash $ncm_file
}

BuildDebPackage() {
    deb_dir=$work_dir/netease-cloud-music

    if [ ! -d "$deb_dir" ];then
        mkdir $deb_dir
    fi

    cd $deb_dir
    cp -rf $NCM_root/debian ./

    # 重新生成control文件
    rm ./debian/control
cat > ./debian/control << EOF
Source: netease-cloud-music
Section: sound
Priority: optional
Maintainer: zccrs <zhangjide@deepin.com>
Standards-Version: 3.9.6
Homepage: https://www.deepin.org

Package: netease-cloud-music
Architecture: any
Description: Netease cloud music player.
 Netease cloud music player.
EOF

    # 重新生成打包规则
    rm ./debian/rules
cat > ./debian/rules << EOF
#!/usr/bin/make -f

%:
	dh \$@ --parallel

override_dh_shlibdeps:

EOF

cat > ./Makefile << EOF
all:

install:
	mkdir -p \$(DESTDIR)
	cp -rfd $output_root/* \$(DESTDIR) 
EOF

    # 将output_root目录中的所有文件构建到deb包
    debuild --no-lintian -us -uc $@
}

InstallDepends && \
BuildQCef && \
BuildNCM && \
CopyDependFiles && \
BuildDebPackage
